@@ -53,26 +53,19 @@
 
 /* Note: we assume there can only be one ALI1535, with one SMBus interface */
 
-#include <linux/version.h>
 #include <linux/module.h>
 #include <linux/pci.h>
-#include <asm/io.h>
-#include <asm/semaphore.h>
 #include <linux/kernel.h>
 #include <linux/stddef.h>
 #include <linux/sched.h>
 #include <linux/ioport.h>
 #include <linux/i2c.h>
-#include "version.h"
 #include <linux/init.h>
+#include <asm/io.h>
+#include <asm/semaphore.h>
+#include "version.h"
+#include "sensors_compat.h"
 
-#ifdef MODULE_LICENSE
-MODULE_LICENSE("GPL");
-#endif
-
-#ifndef DECLARE_MUTEX
-#define DECLARE_MUTEX(name)  struct semaphore name = MUTEX
-#endif /* def DECLARE_MUTEX */
 
 /* ALI1535 SMBus address offsets */
 #define SMBHSTSTS (0 + ali1535_smba)
@@ -144,52 +137,7 @@ MODULE_LICENSE("GPL");
                                         /*  -> Read  = 1               */
 #define	ALI1535_SMBIO_EN	0x04	/* SMB I/O Space enable        */
 
-#ifdef MODULE
-static
-#else
-extern
-#endif
-int __init i2c_ali1535_init(void);
-static int __init ali1535_cleanup(void);
-static int ali1535_setup(void);
-static s32 ali1535_access(struct i2c_adapter *adap, u16 addr,
-			  unsigned short flags, char read_write,
-			  u8 command, int size,
-			  union i2c_smbus_data *data);
-static void ali1535_do_pause(unsigned int amount);
-static int ali1535_transaction(void);
-static void ali1535_inc(struct i2c_adapter *adapter);
-static void ali1535_dec(struct i2c_adapter *adapter);
-static u32 ali1535_func(struct i2c_adapter *adapter);
 
-#ifdef MODULE
-extern int init_module(void);
-extern int cleanup_module(void);
-#endif				/* MODULE */
-
-static struct i2c_algorithm smbus_algorithm = {
-	/* name */ "Non-i2c SMBus adapter",
-	/* id */ I2C_ALGO_SMBUS,
-	/* master_xfer */ NULL, 
-	/* smbus_access */ ali1535_access,
-	/* slave_send */ NULL,
-	/* slave_rcv */ NULL,
-	/* algo_control */ NULL,
-	/* functionality */ ali1535_func,
-};
-
-static struct i2c_adapter ali1535_adapter = {
-	"unset",
-	I2C_ALGO_SMBUS | I2C_HW_SMBUS_ALI1535,
-	&smbus_algorithm,
-	NULL,
-	ali1535_inc,
-	ali1535_dec,
-	NULL,
-	NULL,
-};
-
-static int __initdata ali1535_initialized;
 static unsigned short ali1535_smba = 0;
 DECLARE_MUTEX(i2c_ali1535_sem);
 
@@ -198,32 +146,11 @@ DECLARE_MUTEX(i2c_ali1535_sem);
    Note the differences between kernels with the old PCI BIOS interface and
    newer kernels with the real PCI interface. In compat.h some things are
    defined to make the transition easier. */
-int ali1535_setup(void)
+int ali1535_setup(struct pci_dev *ALI1535_dev)
 {
 	int error_return = 0;
 	unsigned char temp;
 
-	struct pci_dev *ALI1535_dev;
-
-	/* First check whether we can access PCI at all */
-	if (pci_present() == 0) {
-		printk("i2c-ali1535.o: Error: No PCI-bus found!\n");
-		error_return = -ENODEV;
-		goto END;
-	}
-
-	/* Look for the ALI1535, M7101 device */
-	ALI1535_dev = NULL;
-	ALI1535_dev = pci_find_device(PCI_VENDOR_ID_AL,
-				      PCI_DEVICE_ID_AL_M7101, 
-				      ALI1535_dev); 
-
-	if (ALI1535_dev == NULL) {
-		printk("i2c-ali1535.o: Error: Can't detect ali1535!\n");
-		error_return = -ENODEV;
-		goto END;
-	}
-
 /* Check the following things:
 	- SMB I/O address is initialized
 	- Device is enabled
@@ -294,242 +221,122 @@ int ali1535_setup(void)
 }
 
 
-/* Internally used pause function */
-void ali1535_do_pause(unsigned int amount)
+#define MAX_TIMEOUT_HEROS		100
+#define MAX_TIMEOUT_BLOCK_HEROS	500
+#define MAX_TRY_HEROS			3
+
+/* believe it or not this is the delay technique recommended by ALI */
+static void ali1535_delay_loop(void)
 {
-	current->state = TASK_INTERRUPTIBLE;
-	schedule_timeout(amount);
+	int i;
+
+	for(i=0;i<30;i++)
+		outb_p(0,0xEB);
 }
 
-/* Another internally used function */
-int ali1535_transaction(void)
+static int ali1535_wait_for_status(int count,int status)
 {
-	int temp;
-	int result = 0;
-	int timeout = 0;
+	int i;
+	int dat;
 
-#ifdef DEBUG
-	printk
-	    ("i2c-ali1535.o: Transaction (pre): STS=%02x, TYP=%02x, CMD=%02x, ADD=%02x, DAT0=%02x, "
-	     "DAT1=%02x\n", inb_p(SMBHSTSTS), inb_p(SMBHSTTYP),
-	     inb_p(SMBHSTCMD), inb_p(SMBHSTADD), inb_p(SMBHSTDAT0),
-	     inb_p(SMBHSTDAT1));
-#endif
-
-	/* get status */
-	temp = inb_p(SMBHSTSTS);
-
-	/* Make sure the SMBus host is ready to start transmitting */
-	/* Check the busy bit first */
-	if (temp & ALI1535_STS_BUSY) {
-/*
-   If the host controller is still busy, it may have timed out in the previous transaction,
-   resulting in a "SMBus Timeout" printk.
-   I've tried the following to reset a stuck busy bit.
-	1. Reset the controller with an KILL command.
-	   (this doesn't seem to clear the controller if an external device is hung)
-	2. Reset the controller and the other SMBus devices with a T_OUT command.
-	   (this clears the host busy bit if an external device is hung,
-	   but it comes back upon a new access to a device)
-	3. Disable and reenable the controller in SMBHSTCFG
-   Worst case, nothing seems to work except power reset.
-*/
-/* Abort - reset the host controller */
-/*
-#ifdef DEBUG
-    printk("i2c-ali1535.o: Resetting host controller to clear busy condition\n",temp);
-#endif
-    outb_p(ALI1535_KILL, SMBHSTTYP);
-    temp = inb_p(SMBHSTSTS);
-    if (temp & ALI1535_STS_BUSY) {
-*/
-
-/*
-   Try resetting entire SMB bus, including other devices -
-   This may not work either - it clears the BUSY bit but
-   then the BUSY bit may come back on when you try and use the chip again.
-   If that's the case you are stuck.
-*/
-		printk
-		    ("i2c-ali1535.o: Resetting entire SMB Bus to clear busy condition (%02x)\n",
-		     temp);
-		outb_p(ALI1535_T_OUT, SMBHSTTYP);
-		temp = inb_p(SMBHSTSTS);
-	}
-/*
-  }
-*/
-
-	/* now check the error bits and the busy bit */
-	if (temp & (ALI1535_STS_ERR | ALI1535_STS_BUSY)) {
-		/* do a clear-on-write */
-		outb_p(0xFF, SMBHSTSTS);
-		if ((temp = inb_p(SMBHSTSTS)) &
-		    (ALI1535_STS_ERR | ALI1535_STS_BUSY)) {
-			/* this is probably going to be correctable only by a power reset
-			   as one of the bits now appears to be stuck */
-			/* This may be a bus or device with electrical problems. */
-			printk
-			    ("i2c-ali1535.o: SMBus reset failed! (0x%02x) - controller or device on bus is probably hung\n",
-			     temp);
-			return -1;
-		}
-	} else {
-		/* check and clear done bit */
-		if (temp & ALI1535_STS_DONE) {
-			outb_p(temp, SMBHSTSTS);
-		}
+	for (i = 0; i < count; i++) {
+		ali1535_delay_loop();
+		dat = inb_p(SMBHSTSTS);
+		if (dat == status)
+			break;
 	}
 
-	/* start the transaction by writing anything to the start register */
-	outb_p(0xFF, SMBHSTPORT);
-
-	/* We will always wait for a fraction of a second! */
-	timeout = 0;
-	do {
-		ali1535_do_pause(1);
-		temp = inb_p(SMBHSTSTS);
-	} while (((temp & ALI1535_STS_BUSY) && !(temp & ALI1535_STS_IDLE))
-		 && (timeout++ < MAX_TIMEOUT));
-
-	/* If the SMBus is still busy, we give up */
-	if (timeout >= MAX_TIMEOUT) {
-		result = -1;
-		printk("i2c-ali1535.o: SMBus Timeout!\n");
-	}
-
-	if (temp & ALI1535_STS_FAIL) {
-		result = -1;
-#ifdef DEBUG
-		printk("i2c-ali1535.o: Error: Failed bus transaction\n");
-#endif
-	}
-
-/*
-  Unfortunately the ALI SMB controller maps "no response" and "bus collision"
-  into a single bit. No reponse is the usual case so don't do a printk.
-  This means that bus collisions go unreported.
-*/
-	if (temp & ALI1535_STS_BUSERR) {
-		result = -1;
-#ifdef DEBUG
-		printk
-		    ("i2c-ali1535.o: Error: no response or bus collision ADD=%02x\n",
-		     inb_p(SMBHSTADD));
-#endif
-	}
-
-/* haven't ever seen this */
-	if (temp & ALI1535_STS_DEV) {
-		result = -1;
-		printk("i2c-ali1535.o: Error: device error\n");
-	}
-
-/* 
-   check to see if the "command complete" indication is set
- */
-	if (!(temp & ALI1535_STS_DONE)) {
-		result = -1;
-		printk("i2c-ali1535.o: Error: command never completed\n");
-	}
-#ifdef DEBUG
-	printk
-	    ("i2c-ali1535.o: Transaction (post): STS=%02x, TYP=%02x, CMD=%02x, ADD=%02x, "
-	     "DAT0=%02x, DAT1=%02x\n", inb_p(SMBHSTSTS), inb_p(SMBHSTTYP),
-	     inb_p(SMBHSTCMD), inb_p(SMBHSTADD), inb_p(SMBHSTDAT0),
-	     inb_p(SMBHSTDAT1));
-#endif
-
-/* 
-    take consequent actions for error conditions
- */
-        if (!(temp & ALI1535_STS_DONE)) {
-	  /* issue "kill" to reset host controller */
-	  outb_p(ALI1535_KILL,SMBHSTTYP);
-	  outb_p(0xFF,SMBHSTSTS);
-	}	  
-	else if (temp & ALI1535_STS_ERR) {
-	  /* issue "timeout" to reset all devices on bus */
-	  outb_p(ALI1535_T_OUT,SMBHSTTYP);
-	  outb_p(0xFF,SMBHSTSTS);
-	}
-        
-	return result;
+	return i == count ? 1 : 0;
 }
 
-/* Return -1 on error. See smbus.h for more information */
+/* Return -1 on error. */
 s32 ali1535_access(struct i2c_adapter * adap, u16 addr,
 		   unsigned short flags, char read_write, u8 command,
 		   int size, union i2c_smbus_data * data)
 {
 	int i, len;
-	int temp;
-	int timeout;
 	s32 result = 0;
+	int timeout = 0;
+	int oldsize = size;
 
 	down(&i2c_ali1535_sem);
-/* make sure SMBus is idle */
-	temp = inb_p(SMBHSTSTS);
-	for (timeout = 0;
-	     (timeout < MAX_TIMEOUT) && !(temp & ALI1535_STS_IDLE);
-	     timeout++) {
-		ali1535_do_pause(1);
-		temp = inb_p(SMBHSTSTS);
-	}
-	if (timeout >= MAX_TIMEOUT) {
-		printk("i2c-ali1535.o: Idle wait Timeout! STS=0x%02x\n",
-		       temp);
-	}
 
-/* clear status register (clear-on-write) */
-	outb_p(0xFF, SMBHSTSTS);
-
-	switch (size) {
-	case I2C_SMBUS_PROC_CALL:
-		printk
-		    ("i2c-ali1535.o: I2C_SMBUS_PROC_CALL not supported!\n");
+repeat:
+	if(timeout++ > MAX_TRY_HEROS) {
 		result = -1;
 		goto EXIT;
+	}
+
+	/* clear status */
+	outb_p(0xFF, SMBHSTSTS);
+
+	if (ali1535_wait_for_status(MAX_TIMEOUT_HEROS,ALI1535_STS_IDLE))
+		goto repeat;
+
+	outb_p(ALI1535_KILL,SMBHSTTYP);
+
+	if (ali1535_wait_for_status(MAX_TIMEOUT_HEROS,ALI1535_STS_FAIL))
+		goto repeat;
+
+	/* clear status */
+	outb_p(0xFF, SMBHSTSTS);
+
+	if (ali1535_wait_for_status(MAX_TIMEOUT_HEROS,ALI1535_STS_IDLE))
+		goto repeat;
+
+	switch (size) {
 	case I2C_SMBUS_QUICK:
-		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01),
-		       SMBHSTADD);
+		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01), SMBHSTADD);
+		ali1535_delay_loop();
 		size = ALI1535_QUICK;
-                outb_p(size, SMBHSTTYP);	/* output command */
-                break;
+		outb_p(size, SMBHSTTYP);	/* output command */
+		ali1535_delay_loop();
+		break;
 	case I2C_SMBUS_BYTE:
-		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01),
-		       SMBHSTADD);
+		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01), SMBHSTADD);
+		ali1535_delay_loop();
 		size = ALI1535_BYTE;
-                outb_p(size, SMBHSTTYP);	/* output command */
-		if (read_write == I2C_SMBUS_WRITE)
+		outb_p(size, SMBHSTTYP);	/* output command */
+		ali1535_delay_loop();
+		if (read_write == I2C_SMBUS_WRITE) {
 			outb_p(command, SMBHSTCMD);
+			ali1535_delay_loop();
+		}
 		break;
 	case I2C_SMBUS_BYTE_DATA:
-		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01),
-		       SMBHSTADD);
+		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01), SMBHSTADD);
+		ali1535_delay_loop();
 		size = ALI1535_BYTE_DATA;
-                outb_p(size, SMBHSTTYP);	/* output command */
+		outb_p(size, SMBHSTTYP);	/* output command */
+		ali1535_delay_loop();
 		outb_p(command, SMBHSTCMD);
-		if (read_write == I2C_SMBUS_WRITE)
+		ali1535_delay_loop();
+		if (read_write == I2C_SMBUS_WRITE) {
 			outb_p(data->byte, SMBHSTDAT0);
+			ali1535_delay_loop();
+		}
 		break;
 	case I2C_SMBUS_WORD_DATA:
-		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01),
-		       SMBHSTADD);
+		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01), SMBHSTADD);
+		ali1535_delay_loop();
 		size = ALI1535_WORD_DATA;
-                outb_p(size, SMBHSTTYP);	/* output command */
+		outb_p(size, SMBHSTTYP);	/* output command */
 		outb_p(command, SMBHSTCMD);
+		ali1535_delay_loop();
 		if (read_write == I2C_SMBUS_WRITE) {
 			outb_p(data->word & 0xff, SMBHSTDAT0);
+			ali1535_delay_loop();
 			outb_p((data->word & 0xff00) >> 8, SMBHSTDAT1);
+			ali1535_delay_loop();
 		}
 		break;
 	case I2C_SMBUS_BLOCK_DATA:
-		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01),
-		       SMBHSTADD);
+		outb_p(((addr & 0x7f) << 1) | (read_write & 0x01), SMBHSTADD);
+		ali1535_delay_loop();
 		size = ALI1535_BLOCK_DATA;
-                outb_p(size, SMBHSTTYP);	/* output command */
+		outb_p(size, SMBHSTTYP);	/* output command */
 		outb_p(command, SMBHSTCMD);
+		ali1535_delay_loop();
 		if (read_write == I2C_SMBUS_WRITE) {
 			len = data->block[0];
 			if (len < 0) {
@@ -541,43 +348,84 @@ s32 ali1535_access(struct i2c_adapter * adap, u16 addr,
 				data->block[0] = len;
 			}
 			outb_p(len, SMBHSTDAT0);
-			outb_p(inb_p(SMBHSTTYP) | ALI1535_BLOCK_CLR, SMBHSTTYP);	/* Reset SMBBLKDAT */
-			for (i = 1; i <= len; i++)
+			ali1535_delay_loop();
+			{
+				int val;
+
+				val = inb_p(SMBHSTTYP) | ALI1535_BLOCK_CLR;
+				ali1535_delay_loop();
+				outb_p(val, SMBHSTTYP);	/* Reset SMBBLKDAT */
+				ali1535_delay_loop();
+			}
+			for (i = 1; i <= len; i++) {
 				outb_p(data->block[i], SMBBLKDAT);
+				ali1535_delay_loop();
+			}
 		}
 		break;
-	}
-
-	if (ali1535_transaction())	/* Error in transaction */
-	  {
+	default:
+		printk
+		    (KERN_WARNING "i2c-ali1535.o: Unsupported transaction %d\n", size);
 		result = -1;
 		goto EXIT;
-          }
+	}
 
-	if ((read_write == I2C_SMBUS_WRITE) || (size == ALI1535_QUICK))
-	  {
+	/* start the transaction */
+	outb_p(0xFF, SMBHSTPORT);
+	ali1535_delay_loop();
+
+	if (ali1535_wait_for_status(size == ALI1535_BLOCK_DATA ?
+                                            MAX_TIMEOUT_BLOCK_HEROS :
+                                            MAX_TIMEOUT_HEROS ,
+                                       ALI1535_STS_IDLE | ALI1535_STS_DONE)) {
+		size = oldsize;
+		goto repeat;
+	}
+
+	/* clear status */
+	outb_p(0xFF, SMBHSTSTS);
+	ali1535_delay_loop();
+	
+	if(inb_p(SMBHSTSTS) != ALI1535_STS_IDLE) {
+		size = oldsize;
+		goto repeat;
+	}
+
+	if ((read_write == I2C_SMBUS_WRITE) || (size == ALI1535_QUICK)) {
 		result = 0;
 		goto EXIT;
-          }
+        }
 
 	switch (size) {
 	case ALI1535_BYTE:	/* Result put in SMBHSTDAT0 */
 		data->byte = inb_p(SMBHSTDAT0);
+		ali1535_delay_loop();
 		break;
 	case ALI1535_BYTE_DATA:
 		data->byte = inb_p(SMBHSTDAT0);
+		ali1535_delay_loop();
 		break;
 	case ALI1535_WORD_DATA:
 		data->word = inb_p(SMBHSTDAT0) + (inb_p(SMBHSTDAT1) << 8);
+		ali1535_delay_loop();
 		break;
 	case ALI1535_BLOCK_DATA:
 		len = inb_p(SMBHSTDAT0);
+		ali1535_delay_loop();
 		if (len > 32)
 			len = 32;
 		data->block[0] = len;
-		outb_p(inb_p(SMBHSTTYP) | ALI1535_BLOCK_CLR, SMBHSTTYP);	/* Reset SMBBLKDAT */
+		{
+			int val;
+
+			val = inb_p(SMBHSTTYP) | ALI1535_BLOCK_CLR;
+			ali1535_delay_loop();
+			outb_p(val, SMBHSTTYP);	/* Reset SMBBLKDAT */
+			ali1535_delay_loop();
+		}
 		for (i = 1; i <= data->block[0]; i++) {
 			data->block[i] = inb_p(SMBBLKDAT);
+			ali1535_delay_loop();
 #ifdef DEBUG
 			printk
 			    ("i2c-ali1535.o: Blk: len=%d, i=%d, data=%02x\n",
@@ -591,15 +439,18 @@ EXIT:
 	return result;
 }
 
-void ali1535_inc(struct i2c_adapter *adapter)
+static void ali1535_inc(struct i2c_adapter *adapter)
 {
+#ifdef MODULE
 	MOD_INC_USE_COUNT;
+#endif
 }
 
-void ali1535_dec(struct i2c_adapter *adapter)
+static void ali1535_dec(struct i2c_adapter *adapter)
 {
-
+#ifdef MODULE
 	MOD_DEC_USE_COUNT;
+#endif
 }
 
 u32 ali1535_func(struct i2c_adapter *adapter)
@@ -609,82 +460,75 @@ u32 ali1535_func(struct i2c_adapter *adapter)
 	    I2C_FUNC_SMBUS_BLOCK_DATA;
 }
 
-int __init i2c_ali1535_init(void)
+static struct i2c_algorithm smbus_algorithm = {
+	.name		= "Non-i2c SMBus adapter",
+	.id		= I2C_ALGO_SMBUS,
+	.smbus_xfer	= ali1535_access,
+	.functionality	= ali1535_func,
+};
+
+static struct i2c_adapter ali1535_adapter = {
+	.name		= "unset",
+	.id		= I2C_ALGO_SMBUS | I2C_HW_SMBUS_ALI1535,
+	.algo		= &smbus_algorithm,
+	.inc_use	= ali1535_inc,
+	.dec_use	= ali1535_dec,
+};
+
+
+static struct pci_device_id ali1535_ids[] __devinitdata = {
+	{
+	.vendor =	PCI_VENDOR_ID_AL,
+	.device =	PCI_DEVICE_ID_AL_M7101,
+	.subvendor =	PCI_ANY_ID,
+	.subdevice =	PCI_ANY_ID,
+	},
+	{ 0, }
+};
+
+static int __devinit ali1535_probe(struct pci_dev *dev, const struct pci_device_id *id)
 {
-	int res;
-	printk("i2c-ali1535.o version %s (%s)\n", LM_VERSION, LM_DATE);
-#ifdef DEBUG
-/* PE- It might be good to make this a permanent part of the code! */
-	if (ali1535_initialized) {
-		printk
-		    ("i2c-ali1535.o: Oops, ali1535_init called a second time!\n");
-		return -EBUSY;
-	}
-#endif
-	ali1535_initialized = 0;
-	if ((res = ali1535_setup())) {
+	if (ali1535_setup(dev)) {
 		printk
 		    ("i2c-ali1535.o: ALI1535 not detected, module not inserted.\n");
-		ali1535_cleanup();
-		return res;
+		return -ENODEV;
 	}
-	ali1535_initialized++;
+
 	sprintf(ali1535_adapter.name, "SMBus ALI1535 adapter at %04x",
 		ali1535_smba);
-	if ((res = i2c_add_adapter(&ali1535_adapter))) {
-		printk
-		    ("i2c-ali1535.o: Adapter registration failed, module not inserted.\n");
-		ali1535_cleanup();
-		return res;
-	}
-	ali1535_initialized++;
-	printk
-	    ("i2c-ali1535.o: ALI1535 SMBus Controller detected and initialized\n");
-	return 0;
+	return i2c_add_adapter(&ali1535_adapter);
 }
 
-int __init ali1535_cleanup(void)
+static void __devexit ali1535_remove(struct pci_dev *dev)
 {
-	int res;
-	if (ali1535_initialized >= 2) {
-		if ((res = i2c_del_adapter(&ali1535_adapter))) {
-			printk
-			    ("i2c-ali1535.o: i2c_del_adapter failed, module not removed\n");
-			return res;
-		} else
-			ali1535_initialized--;
-	}
-	if (ali1535_initialized >= 1) {
-		release_region(ali1535_smba, ALI1535_SMB_IOSIZE);
-		ali1535_initialized--;
-	}
-	return 0;
+	i2c_del_adapter(&ali1535_adapter);
+	release_region(ali1535_smba, ALI1535_SMB_IOSIZE);
 }
 
-#ifdef RLX
-EXPORT_SYMBOL(ali1535_smba);
-EXPORT_SYMBOL(ali1535_access);
-EXPORT_SYMBOL(i2c_ali1535_sem);
-#else
-EXPORT_NO_SYMBOLS;
-#endif
 
-#ifdef MODULE
+static struct pci_driver ali1535_driver = {
+	.name		= "ali1535 smbus",
+	.id_table	= ali1535_ids,
+	.probe		= ali1535_probe,
+	.remove		= __devexit_p(ali1535_remove),
+};
+
+static int __init i2c_ali1535_init(void)
+{
+	printk("i2c-ali1535.o version %s (%s)\n", LM_VERSION, LM_DATE);
+	return pci_module_init(&ali1535_driver);
+}
+
+static void __exit i2c_ali1535_exit(void)
+{
+	pci_unregister_driver(&ali1535_driver);
+}
 
 MODULE_AUTHOR
-    ("Frodo Looijaard <frodol@dds.nl>, Philip Edelbrock <phil@netroedge.com>,
-      Mark D. Studebaker <mdsxyz123@yahoo.com> and Dan Eaton <dan.eaton@rocketlogix.com>");
+    ("Frodo Looijaard <frodol@dds.nl>, Philip Edelbrock <phil@netroedge.com>, "
+     "Mark D. Studebaker <mdsxyz123@yahoo.com> and Dan Eaton <dan.eaton@rocketlogix.com>");
 MODULE_DESCRIPTION("ALI1535 SMBus driver");
+MODULE_LICENSE("GPL");
 
-int init_module(void)
-{
-	return i2c_ali1535_init();
-}
-
-int cleanup_module(void)
-{
-	return ali1535_cleanup();
-}
-
-#endif				/* MODULE */
-
+module_init(i2c_ali1535_init);
+module_exit(i2c_ali1535_exit);
