@@ -294,6 +294,20 @@ static s32 amd756_access(struct i2c_adapter * adap, u16 addr,
 	return 0;
 }
 
+static void amd756_inc(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_INC_USE_COUNT;
+#endif
+}
+
+static void amd756_dec(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_DEC_USE_COUNT;
+#endif
+}
+
 static u32 amd756_func(struct i2c_adapter *adapter)
 {
 	return I2C_FUNC_SMBUS_QUICK | I2C_FUNC_SMBUS_BYTE |
@@ -308,14 +322,19 @@ static struct i2c_algorithm smbus_algorithm = {
 	.functionality	= amd756_func,
 };
 
-static struct i2c_adapter amd756_adapter = {
-	.owner		= THIS_MODULE,
+struct i2c_adapter amd756_smbus = {
 	.name		= "unset",
 	.id		= I2C_ALGO_SMBUS | I2C_HW_SMBUS_AMD756,
 	.algo		= &smbus_algorithm,
+	.inc_use	= amd756_inc,
+	.dec_use	= amd756_dec,
 };
 
 enum chiptype { AMD756, AMD766, AMD768, NFORCE, AMD8111 };
+static const char* chipname[] = {
+	"AMD756", "AMD766", "AMD768",
+	"nVidia nForce", "AMD8111",
+};
 
 static struct pci_device_id amd756_ids[] __devinitdata = {
 	{PCI_VENDOR_ID_AMD, 0x740B, PCI_ANY_ID, PCI_ANY_ID, 0, 0, AMD756 },
@@ -375,10 +394,10 @@ static int __devinit amd756_probe(struct pci_dev *pdev,
 	printk(KERN_DEBUG DRV_NAME ": AMD756_smba = 0x%X\n", amd756_ioport);
 #endif
 
-	sprintf(amd756_adapter.name,
-		"SMBus AMD756 adapter at %04x", amd756_ioport);
+	sprintf(amd756_smbus.name, "SMBus %s adapter at %04x",
+		chipname[id->driver_data], amd756_ioport);
 
-	error = i2c_add_adapter(&amd756_adapter);
+	error = i2c_add_adapter(&amd756_smbus);
 	if (error) {
 		printk(KERN_ERR DRV_NAME
 		       ": Adapter registration failed, module not inserted.\n");
@@ -412,7 +431,7 @@ static int __init i2c_amd756_init(void)
 
 static void __exit i2c_amd756_exit(void)
 {
-	i2c_del_adapter(&amd756_adapter);
+	i2c_del_adapter(&amd756_smbus);
 	release_region(amd756_ioport, SMB_IOSIZE);
 }
 
@@ -420,5 +439,7 @@ MODULE_AUTHOR("Merlin Hughes <merlin@merlin.org>");
 MODULE_DESCRIPTION("AMD756/766/768/8111 and nVidia nForce SMBus driver");
 MODULE_LICENSE("GPL");
 
+EXPORT_SYMBOL(amd756_smbus);
+
 module_init(i2c_amd756_init)
 module_exit(i2c_amd756_exit)
