@@ -29,12 +29,14 @@
     82801DB		24C3   (HW PEC supported, 32 byte buffer not supported)
     82801EB		24D3   (HW PEC supported, 32 byte buffer not supported)
     6300ESB		25A4   ("")
-    ICH6		266A
+    ICH6		266A   ("")
+    ICH7		27DA   ("")
     This driver supports several versions of Intel's I/O Controller Hubs (ICH).
     For SMBus support, they are similar to the PIIX4 and are part
     of Intel's '810' and other chipsets.
     See the doc/busses/i2c-i801 file for details.
-    I2C Block Read and Process Call are not supported.
+    I2C Block Read supported for ICH5 and higher.
+    Block Process Call are not supported.
 */
 
 /* Note: we assume there can only be one I801, with one SMBus interface */
@@ -53,9 +55,34 @@
 #include "version.h"
 #include "sensors_compat.h"
 
+/* 82801CA is undefined before kernel 2.4.13 */
+#ifndef PCI_DEVICE_ID_INTEL_82801CA_3
+#define PCI_DEVICE_ID_INTEL_82801CA_3	0x2483
+#endif
+
 /* 82801DB is undefined before kernel 2.4.19 */
 #ifndef PCI_DEVICE_ID_INTEL_82801DB_3
-#define PCI_DEVICE_ID_INTEL_82801DB_3      0x24c3
+#define PCI_DEVICE_ID_INTEL_82801DB_3	0x24c3
+#endif
+
+/* 82801EB is undefined before kernel 2.4.21 */
+#ifndef PCI_DEVICE_ID_INTEL_82801EB_3
+#define PCI_DEVICE_ID_INTEL_82801EB_3	0x24d3
+#endif
+
+/* ESB is undefined before kernel 2.4.22 */
+#ifndef PCI_DEVICE_ID_INTEL_ESB_4
+#define PCI_DEVICE_ID_INTEL_ESB_4	0x25a4
+#endif
+
+/* ICH6 is undefined */
+#ifndef PCI_DEVICE_ID_INTEL_ICH6_16
+#define PCI_DEVICE_ID_INTEL_ICH6_16	0x266a
+#endif
+
+/* ICH7 is undefined */
+#ifndef PCI_DEVICE_ID_INTEL_ICH7_17
+#define PCI_DEVICE_ID_INTEL_ICH7_17	0x27da
 #endif
 
 #ifdef I2C_FUNC_SMBUS_BLOCK_DATA_PEC
@@ -95,11 +122,11 @@
 #define I801_WORD_DATA		0x0C
 #define I801_PROC_CALL		0x10	/* later chips only, unimplemented */
 #define I801_BLOCK_DATA		0x14
-#define I801_I2C_BLOCK_DATA	0x18	/* unimplemented */
+#define I801_I2C_BLOCK_DATA	0x18	/* ich4 and later */
 #define I801_BLOCK_LAST		0x34
 #define I801_I2C_BLOCK_LAST	0x38	/* unimplemented */
 #define I801_START		0x40
-#define I801_PEC_EN		0x80	/* ICH4 only */
+#define I801_PEC_EN		0x80	/* ich4 and later */
 
 /* insmod parameters */
 
@@ -117,7 +144,8 @@ static int i801_block_transaction(union i2c_smbus_data *data,
 
 static unsigned short i801_smba;
 static struct pci_dev *I801_dev;
-static int isich4;
+static int isich4;	/* is PEC supported? */
+static int isich5;	/* is i2c block read supported? */
 
 static int i801_setup(struct pci_dev *dev)
 {
@@ -130,11 +158,14 @@ static int i801_setup(struct pci_dev *dev)
 
 	I801_dev = dev;
 	if (dev->device == PCI_DEVICE_ID_INTEL_82801DB_3 ||
-	    dev->device == 0x24d3 ||
-	    dev->device == 0x25a4)
+	    dev->device == PCI_DEVICE_ID_INTEL_82801EB_3 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ESB_4 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ICH6_16 ||
+	    dev->device == PCI_DEVICE_ID_INTEL_ICH7_17)
 		isich4 = 1;
 	else
 		isich4 = 0;
+	isich5 = isich4 && dev->device != PCI_DEVICE_ID_INTEL_82801DB_3;
 
 	/* Determine the address of the SMBus areas */
 	if (force_addr) {
@@ -274,9 +305,9 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 			pci_read_config_byte(I801_dev, SMBHSTCFG, &hostc);
 			pci_write_config_byte(I801_dev, SMBHSTCFG,
 					      hostc | SMBHSTCFG_I2C_EN);
-		} else {
+		} else if (!isich5) {
 			dev_err(I801_dev,
-				"I2C_SMBUS_I2C_BLOCK_READ not DB!\n");
+				"I2C_SMBUS_I2C_BLOCK_READ unsupported!\n");
 			return -1;
 		}
 	}
@@ -300,6 +331,9 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 	for (i = 1; i <= len; i++) {
 		if (i == len && read_write == I2C_SMBUS_READ)
 			smbcmd = I801_BLOCK_LAST;
+		else if (command == I2C_SMBUS_I2C_BLOCK_DATA &&
+		         read_write == I2C_SMBUS_READ)
+			smbcmd = I801_I2C_BLOCK_DATA;
 		else
 			smbcmd = I801_BLOCK_DATA;
 		outb_p(smbcmd | ENABLE_INT9, SMBHSTCNT);
@@ -368,12 +402,17 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 		}
 
 		if (i == 1 && read_write == I2C_SMBUS_READ) {
-			len = inb_p(SMBHSTDAT0);
-			if (len < 1)
-				len = 1;
-			if (len > 32)
-				len = 32;
-			data->block[0] = len;
+			if (command != I2C_SMBUS_I2C_BLOCK_DATA) {
+				len = inb_p(SMBHSTDAT0);
+				if (len < 1)
+					len = 1;
+				if (len > 32)
+					len = 32;
+				data->block[0] = len;
+			} else {
+				/* if slave returns < 32 bytes transaction will fail */
+				data->block[0] = 32;
+			}
 		}
 
 		/* Retrieve/store value in SMBBLKDAT */
@@ -416,7 +455,8 @@ static int i801_block_transaction(union i2c_smbus_data *data, char read_write,
 #endif
 	result = 0;
 END:
-	if (command == I2C_SMBUS_I2C_BLOCK_DATA) {
+	if (command == I2C_SMBUS_I2C_BLOCK_DATA &&
+	    read_write == I2C_SMBUS_WRITE) {
 		/* restore saved configuration register value */
 		pci_write_config_byte(I801_dev, SMBHSTCFG, hostc);
 	}
@@ -527,6 +567,19 @@ static s32 i801_access(struct i2c_adapter * adap, u16 addr,
 	return 0;
 }
 
+static void i801_inc(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_INC_USE_COUNT;
+#endif
+}
+
+static void i801_dec(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_DEC_USE_COUNT;
+#endif
+}
 
 static u32 i801_func(struct i2c_adapter *adapter)
 {
@@ -537,6 +590,10 @@ static u32 i801_func(struct i2c_adapter *adapter)
 	     | (isich4 ? I2C_FUNC_SMBUS_BLOCK_DATA_PEC |
 	                 I2C_FUNC_SMBUS_HWPEC_CALC
 	               : 0)
+#endif
+#if 0
+	     | (isich5 ? I2C_FUNC_SMBUS_READ_I2C_BLOCK
+	               : 0)
 #endif
 	    ;
 }
@@ -549,10 +606,11 @@ static struct i2c_algorithm smbus_algorithm = {
 };
 
 static struct i2c_adapter i801_adapter = {
-	.owner		= THIS_MODULE,
 	.id		= I2C_ALGO_SMBUS | I2C_HW_SMBUS_I801,
 	.algo		= &smbus_algorithm,
-	.name	= "unset",
+	.name		= "unset",
+	.inc_use	= i801_inc,
+	.dec_use	= i801_dec,
 };
 
 static struct pci_device_id i801_ids[] __devinitdata = {
@@ -588,22 +646,28 @@ static struct pci_device_id i801_ids[] __devinitdata = {
 	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x24d3,	/* 82801EB ICH5 */
+		.device =	PCI_DEVICE_ID_INTEL_82801EB_3,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice =	PCI_ANY_ID,
 	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x25a4, /* PCI_DEVICE_ID_INTEL_ESB_4 */
+		.device =	PCI_DEVICE_ID_INTEL_ESB_4,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice = 	PCI_ANY_ID,
 	},
 	{
 		.vendor =	PCI_VENDOR_ID_INTEL,
-		.device =	0x266a, /* PCI_DEVICE_ID_INTEL_ICH6_16 */
+		.device =	PCI_DEVICE_ID_INTEL_ICH6_16,
 		.subvendor =	PCI_ANY_ID,
 		.subdevice =	PCI_ANY_ID,
   	},
+	{
+		.vendor =	PCI_VENDOR_ID_INTEL,
+		.device =	PCI_DEVICE_ID_INTEL_ICH7_17,
+		.subvendor =	PCI_ANY_ID,
+		.subdevice =	PCI_ANY_ID,
+	},
 	{ 0, }
 };
 
