@@ -156,10 +156,7 @@ static void adm1025_vid(struct i2c_client *client, int operation,
 static void adm1025_vrm(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 
-static int adm1025_id = 0;
-
 static struct i2c_driver adm1025_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "ADM1025 sensor driver",
 	.id		= I2C_DRIVERID_ADM1025,
 	.flags		= I2C_DF_NOTIFY,
@@ -324,8 +321,6 @@ static int adm1025_detect(struct i2c_adapter *adapter, int address,
 	/* Fill in the remaining client fields and put it into the global list */
 	strcpy(new_client->name, client_name);
 	data->type = kind;
-
-	new_client->id = adm1025_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -336,7 +331,8 @@ static int adm1025_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client,
 					type_name,
-					adm1025_dir_table_template)) < 0) {
+					adm1025_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
