@@ -37,7 +37,6 @@
                  Define addresses as a range.
 */
 
-#include <linux/version.h>
 #include <linux/module.h>
 #include <linux/slab.h>
 #include <linux/i2c.h>
@@ -385,7 +384,6 @@ static void adm1026_afc(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 
 static struct i2c_driver adm1026_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "ADM1026 compatible sensor driver",
 	.id		= I2C_DRIVERID_ADM1026,
 	.flags		= I2C_DF_NOTIFY,
@@ -691,7 +689,6 @@ int adm1026_detect(struct i2c_adapter *adapter, int address,
 				" found at %d,0x%02x. Try force_adm1026.\n",
 				i2c_adapter_id(adapter), address );
 			}
-			err = 0 ;
 			goto ERROR1;
 		}
 	}
@@ -754,7 +751,8 @@ int adm1026_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client,
 					type_name,
-					template)) < 0) {
+					template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR2;
 	}
