@@ -196,15 +196,12 @@ static void adm1031_update_client(struct i2c_client *client);
 
 /* This is the driver that will be inserted */
 static struct i2c_driver adm1031_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "ADM1031/ADM1030 sensor driver",
 	.flags		= I2C_DF_NOTIFY,
 	.attach_adapter	= adm1031_attach_adapter,
 	.detach_client	= adm1031_detach_client,
 };
 
-static int adm1031_id = 0;
-
 #define TEMP_TO_REG(val)		((val) < 0 ? (((val) - 500) / 1000) : \
 					 (((val) + 500) / 1000))
 
@@ -286,8 +283,6 @@ adm1031_detect(struct i2c_adapter *adapter, int address,
 	data->chip_type = kind;
 
 	strcpy(new_client->name, client_name);
-
-	new_client->id = adm1031_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -298,11 +293,13 @@ adm1031_detect(struct i2c_adapter *adapter, int address,
 
 	if (kind == adm1030) {
 		if ((err = i2c_register_entry(new_client, type_name,
-					      adm1030_dir_table_template)) < 0)
+					      adm1030_dir_table_template,
+					      THIS_MODULE)) < 0)
 			goto exit_detach;
 	} else if (kind == adm1031) {
 		if ((err = i2c_register_entry(new_client, type_name,
-					      adm1031_dir_table_template)) < 0)
+					      adm1031_dir_table_template,
+					      THIS_MODULE)) < 0)
 			goto exit_detach;
 	}
 
