@@ -82,7 +82,6 @@ static void bmcsensors_get_sdr(u16 resid, u16 record, u8 offset);
 static void bmcsensors_get_reading(struct i2c_client *client, int i);
 
 static struct i2c_driver bmcsensors_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "BMC Sensors driver",
 	.id		= I2C_DRIVERID_BMCSENSORS,
 	.flags		= I2C_DF_NOTIFY,
@@ -153,7 +152,8 @@ static ctl_table *bmcsensors_dir_table;
 static char *bmcsensors_proc_name_pool;
 
 #define IPMI_SDR_SIZE 67
-static int ipmi_sdr_partial_size = IPMI_SDR_SIZE;
+#define IPMI_CHUNK_SIZE 16
+static int ipmi_sdr_partial_size = IPMI_CHUNK_SIZE;
 static struct ipmi_msg tx_message;	/* send message */
 static unsigned char tx_msg_data[IPMI_MAX_MSG_LENGTH + 50];
 static unsigned char rx_msg_data[IPMI_MAX_MSG_LENGTH + 50]; /* sloppy */
@@ -249,6 +249,7 @@ static void bmcsensors_select_thresholds(struct sdrdata * sd)
 {
 	u8 capab = sd->capab;
 	u16 mask = sd->thresh_mask;
+	int tmp;
 
 	sd->lim1 = -1;
 	sd->lim2 = -1;
@@ -296,7 +297,13 @@ static void bmcsensors_select_thresholds(struct sdrdata * sd)
 		}
 	}
 
-	/* fixme swap lim1/lim2 if m < 0 */
+	/* swap lim1/lim2 if m < 0 or function is 1/x (but not both!) */
+	if(sd->m < 0 && sd->linear != 7 || sd->m >= 0 && sd->linear == 7) {
+		tmp = sd->lim1;
+		sd->lim1 = sd->lim2;
+		sd->lim2 = tmp;
+	}
+
 	if(sd->lim1 >= 0)
 		printk(KERN_INFO "bmcsensors.o: using %s for upper limit\n",
 			threshold_text[sd->lim1]);
@@ -380,7 +387,7 @@ static void bmcsensors_build_proc_table()
 				bmcsensors_dir_table[i].procname, id);
 		}
 		bmcsensors_select_thresholds(sdrd + i);
-		if(sdrd[i].linear != 0) {
+		if(sdrd[i].linear != 0 && sdrd[i].linear != 7) {
 			printk(KERN_INFO
 			       "bmcsensors.o: sensor %d: nonlinear function 0x%.2x unsupported, expect bad results\n",
 			       i, sdrd[i].linear);
@@ -402,7 +409,8 @@ static void bmcsensors_build_proc_table()
 	bmcsensors_dir_table[sdrd_count].ctl_name = 0;
 
 	if ((i = i2c_register_entry(&bmc_client, "bmc",
-				    bmcsensors_dir_table)) < 0) {
+				    bmcsensors_dir_table,
+				    THIS_MODULE)) < 0) {
 		printk(KERN_INFO "bmcsensors.o: i2c registration failed.\n");
 		kfree(bmcsensors_dir_table);
 		kfree(bmcsensors_proc_name_pool);
@@ -456,6 +464,7 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 	int id_length;
 	int i;
 	int rstate = STATE_SDR;
+	int ipmi_ver = 0;
 	unsigned char * data;
 	u8 id[SDR_MAX_UNPACKED_ID_LENGTH];
 
@@ -509,7 +518,7 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 		entity = data[11];
 		init = data[13];
 */
-		stype = data[15];
+		stype = data[(ipmi_ver == 0x90?16:15)];
 		if(stype <= STYPE_MAX) {	/* known sensor type */
 			if(bmcs_count[stype] >= bmcs_max[stype]) {
 				if(bmcs_max[stype] > 0)
@@ -527,11 +536,11 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 				       "bmcsensors.o: Limit of %d exceeded for total sensors\n",
 				       MAX_SDR_ENTRIES);
 				nextrecord = 0xffff;
-			} else if(data[16] != 0x01) {
+			} else if(data[(ipmi_ver == 0x90?17:16)] != 0x01) {
 				if(type == 1)
 					ipmi_sprintf(id, &data[51], data[50] >> 6, data[50] & 0x1f);
 				else
-					ipmi_sprintf(id, &data[35], data[34] >> 6, data[34] & 0x1f);
+					ipmi_sprintf(id, &data[(ipmi_ver == 0x90?30:35)], data[(ipmi_ver == 0x90?29:34)] >> 6, data[(ipmi_ver == 0x90?29:34)] & 0x1f);
 				printk(KERN_INFO
 				       "bmcsensors.o: skipping non-threshold sensor \"%s\"\n",
 				       id);
@@ -539,23 +548,23 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 				/* add entry to sdrd table */
 				sdrd[sdrd_count].stype = stype;
 				sdrd[sdrd_count].number = data[10];
-				sdrd[sdrd_count].capab = data[14];
-				sdrd[sdrd_count].thresh_mask = (((u16) data[22]) << 8) | data[21];
+				sdrd[sdrd_count].capab = data[(ipmi_ver == 0x90?15:14)];
+				sdrd[sdrd_count].thresh_mask = (((u16) data[(ipmi_ver == 0x90?21:22)]) << 8) | data[21];
 				if(type == 1) {
-					sdrd[sdrd_count].format = data[24] >> 6;
-					sdrd[sdrd_count].linear = data[26] & 0x7f;
-					sdrd[sdrd_count].m = data[27];
-					sdrd[sdrd_count].m |= ((u16) (data[28] & 0xc0)) << 2;
+					sdrd[sdrd_count].format = data[(ipmi_ver == 0x90?22:24)] >> 6;
+					sdrd[sdrd_count].linear = data[(ipmi_ver == 0x90?25:26)] & 0x7f;
+					sdrd[sdrd_count].m = data[(ipmi_ver == 0x90?26:27)];
+					sdrd[sdrd_count].m |= ((u16) (data[(ipmi_ver == 0x90?27:28)] & 0xc0)) << 2;
 					if(sdrd[sdrd_count].m & 0x0200)
 						sdrd[sdrd_count].m |= 0xfc00;	/* sign extend */
-					sdrd[sdrd_count].b = data[29];
-					sdrd[sdrd_count].b |= ((u16) (data[30] & 0xc0)) << 2;
+					sdrd[sdrd_count].b = data[(ipmi_ver == 0x90?28:29)];
+					sdrd[sdrd_count].b |= ((u16) (data[(ipmi_ver == 0x90?29:30)] & 0xc0)) << 2;
 					if(sdrd[sdrd_count].b & 0x0200)
 						sdrd[sdrd_count].b |= 0xfc00;	/* sign extend */
-					sdrd[sdrd_count].k = data[32];
-					sdrd[sdrd_count].nominal = data[34];
+					sdrd[sdrd_count].k = data[(ipmi_ver == 0x90?31:32)];
+					sdrd[sdrd_count].nominal = data[(ipmi_ver == 0x90?33:34)];
 					for(i = 0; i < SDR_LIMITS; i++)		/* assume readable */
-						sdrd[sdrd_count].limits[i] = data[39 + i];
+						sdrd[sdrd_count].limits[i] = data[(ipmi_ver == 0x90?40:39) + i];
 					sdrd[sdrd_count].string_type = data[50] >> 6;
 					id_length = data[50] & 0x1f;
 					memcpy(sdrd[sdrd_count].id, &data[51], id_length);
@@ -564,12 +573,16 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 					sdrd[sdrd_count].m = 1;
 					sdrd[sdrd_count].b = 0;
 					sdrd[sdrd_count].k = 0;
-					sdrd[sdrd_count].string_type = data[34] >> 6;
+					sdrd[sdrd_count].string_type = data[(ipmi_ver == 0x90?29:34)] >> 6;
 					id_length = data[34] & 0x1f;
 					if(id_length > 0)
-						memcpy(sdrd[sdrd_count].id, &data[35], id_length);
+						memcpy(sdrd[sdrd_count].id, &data[(ipmi_ver == 0x90?30:35)], id_length);
 					sdrd[sdrd_count].id_length = id_length;
 					/* limits?? */
+					if(ipmi_ver == 0x90){
+						memcpy(sdrd[sdrd_count].id, &data[30], id_length);
+						sdrd[sdrd_count].id_length = id_length;
+					}
 				}
 				bmcs_count[stype]++;
 				sdrd_count++;
@@ -604,7 +617,12 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 	if (nextrecord>=6224) {
 	  nextrecord = 0xffff; /*YJ stop sensor scan on poweredge 1750 */
 	}
-			
+	if (ipmi_ver != 0x90) {
+ 		if (nextrecord>=6224) {
+ 			nextrecord = 0xffff; /*YJ stop sensor scan on poweredge 1750 */
+ 		}
+  	}
+		
 	if(nextrecord == 0xFFFF) {
 		if(sdrd_count == 0) {
 			printk(KERN_INFO "bmcsensors.o: No recognized sensors found.\n");
@@ -814,7 +832,7 @@ static int bmcsensors_detect(struct i2c_adapter *adapter, int address,
 	receive_counter = 0;
 	rx_msg_data_offset = 0;
 	errorcount = 0;
-	ipmi_sdr_partial_size = IPMI_SDR_SIZE;
+	ipmi_sdr_partial_size = IPMI_CHUNK_SIZE;
 	for(i = 0; i <= STYPE_MAX; i++)
 		bmcs_count[i] = 0;
 
@@ -926,6 +944,7 @@ static int decplaces(struct sdrdata *sd)
 }
 
 /* convert a raw value to a reading. IMPI V1.5 Section 30 */
+/* 1/x is the only "linearization function" supported */
 static long conv_val(int value, struct sdrdata *sd)
 {
 	u8 k1, k2;
@@ -939,10 +958,23 @@ static long conv_val(int value, struct sdrdata *sd)
 	else
 		r += sd->b / exps[16 - k1];
 	r *= exps[decplaces(sd)];
-	if(k2 < 8)
-		r *= exps[k2];
-	else
-		r /= exps[16 - k2];
+	if(k2 < 8) {
+		if(sd->linear != 7)
+			r *= exps[k2];
+		else
+			// this will always truncate to 0: r = 1 / (exps[k2] * r);
+			r = 0;
+	} else {
+		if(sd->linear != 7)
+			r /= exps[16 - k2];
+		else {
+			if(r != 0)
+				// 1 / x * 10 ** (-m) == 10 ** m / x
+				r = exps[16 - k2] / r;
+			else
+				r = 0;
+		}
+	}
 	return r;
 }
 
