@@ -174,7 +174,6 @@ static void gl518_iterate(struct i2c_client *client, int operation,
 
 /* This is the driver that will be inserted */
 static struct i2c_driver gl518_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "GL518SM sensor chip driver",
 	.id		= I2C_DRIVERID_GL518,
 	.flags		= I2C_DF_NOTIFY,
@@ -346,7 +345,6 @@ static int gl518_detect(struct i2c_adapter *adapter, int address,
 		goto ERROR2;
 	}
 	gl518_list[i] = new_client;
-	new_client->id = i;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -357,7 +355,8 @@ static int gl518_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					gl518_dir_table_template)) < 0) {
+					gl518_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
