@@ -35,12 +35,21 @@
  * Among others, it has a higher accuracy than the LM90, much like the
  * LM86 does.
  *
- * This driver also supports the MAX6657 and MAX6658, sensor chips made
- * by Maxim. These chips are similar to the LM86. Complete datasheet
- * can be obtained at Maxim's website at:
+ * This driver also supports the MAX6657, MAX6658 and MAX6659 sensor
+ * chips made by Maxim. These chips are similar to the LM86. Complete
+ * datasheet can be obtained at Maxim's website at:
  *   http://www.maxim-ic.com/quick_view2.cfm/qv_pk/2578
- * Note that there is no way to differenciate between both chips (but
- * no need either).
+ * Note that there is no easy way to differenciate between the three
+ * variants. The extra address and features of the MAX6659 are not
+ * supported by this driver.
+ *
+ * This driver also supports the ADT7461 chip from Analog Devices but
+ * only in its "compatability mode". If an ADT7461 chip is found but
+ * is configured in non-compatible mode (where its temperature
+ * register values are decoded differently) it is ignored by this
+ * driver. Complete datasheet can be obtained from Analog's website
+ * at:
+ *   http://products.analog.com/products/info.asp?product=ADT7461
  *
  * Since the LM90 was the first chipset supported by this driver, most
  * comments will refer to this chipset, but are actually general and
@@ -74,9 +83,12 @@
 
 /*
  * Addresses to scan
- * Address is fully defined internally and cannot be changed.
+ * Address is fully defined internally and cannot be changed except for
+ * MAX6659.
  * LM86, LM89, LM90, LM99, ADM1032, MAX6657 and MAX6658 have address 0x4c.
  * LM89-1, and LM99-1 have address 0x4d.
+ * MAX6659 can have address 0x4c, 0x4d or 0x4e (unsupported).
+ * ADT7461 always has address 0x4c.
  */
 
 static unsigned short normal_i2c[] = { 0x4c, 0x4d, SENSORS_I2C_END };
@@ -88,7 +100,7 @@ static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
  * Insmod parameters
  */
 
-SENSORS_INSMOD_5(lm90, adm1032, lm99, lm86, max6657);
+SENSORS_INSMOD_6(lm90, adm1032, lm99, lm86, max6657, adt7461);
 
 /*
  * The LM90 registers
@@ -131,19 +143,33 @@ SENSORS_INSMOD_5(lm90, adm1032, lm99, lm86, max6657);
 
 /*
  * Conversions and various macros
- * The LM90 uses signed 8-bit values for the local temperatures,
- * and signed 11-bit values for the remote temperatures (except
- * T_CRIT). The 11-bit conversion formulas may not round negative
- * numbers perfectly, but who cares?
+ * For local temperatures and limits, critical limits and the hysteresis
+ * value, the LM90 uses signed 8-bit values with LSB = 1 degree Celcius.
+ * For remote temperatures and limits, it uses signed 11-bit values with
+ * LSB = 0.125 degree Celcius, left-justified in 16-bit registers.
  */
 
-#define TEMP1_FROM_REG(val)  (val & 0x80 ? val-0x100 : val)
-#define TEMP1_TO_REG(val)    (val < 0 ? val+0x100 : val)
-#define TEMP2_FROM_REG(val)  (((val & 0x8000 ? val-0x10000 : val) \
-                             * 10 + 128) >> 8)
-#define TEMP2_TO_REG(val)    (((val << 8) / 10 + (val < 0 ? \
-                             0x10000 : 0)) & 0xFFE0)
-#define HYST_TO_REG(val)     (val < 0 ? 0 : val > 31 ? 31 : val)
+#define TEMP1_FROM_REG(val)	(val)
+#define TEMP1_TO_REG(val)	((val) <= -128 ? -128 : \
+				 (val) >= 127 ? 127 : (val))
+#define TEMP2_FROM_REG(val)	((val) / 32 * 125 / 100)
+#define TEMP2_TO_REG(val)	((val) <= -1280 ? 0x8000 : \
+				 (val) >= 1270 ? 0x7FE0 : \
+				 ((val) * 100 / 125 * 32))
+#define HYST_TO_REG(val)	((val) <= 0 ? 0 : \
+				 (val) >= 31 ? 31 : (val))
+
+/* 
+ * ADT7461 is almost identical to LM90 except that attempts to write
+ * values that are outside the range 0 < temp < 127 are treated as
+ * the boundary value. 
+ */
+
+#define TEMP1_TO_REG_ADT7461(val) ((val) <= 0 ? 0 : \
+				 (val) >= 127 ? 127 : (val))
+#define TEMP2_TO_REG_ADT7461(val) ((val) <= 0 ? 0 : \
+				 (val) >= 1277 ? 0x7FC0 : \
+				 ((val) * 100 / 250 * 64))
 
 /*
  * Functions declaration
@@ -174,7 +200,6 @@ static void lm90_alarms(struct i2c_client *client, int operation,
  */
 
 static struct i2c_driver lm90_driver = {
-	.owner          = THIS_MODULE,
 	.name           = "LM90/ADM1032 sensor driver",
 	.id             = I2C_DRIVERID_LM90,
 	.flags          = I2C_DF_NOTIFY,
@@ -194,13 +219,14 @@ struct lm90_data
 	struct semaphore update_lock;
 	char valid; /* zero until following fields are valid */
 	unsigned long last_updated; /* in jiffies */
+	int kind;
 
 	/* registers values */
-	u8 local_temp, local_high, local_low;
-	u16 remote_temp, remote_high, remote_low; /* combined */
-	u8 local_crit, remote_crit;
+	s8 local_temp, local_high, local_low;
+	s16 remote_temp, remote_high, remote_low; /* combined */
+	s8 local_crit, remote_crit;
 	u8 hyst; /* linked to two sysctl files (hyst1 RW, hyst2 RO) */
-	u16 alarms; /* bitvector, combined */
+	u8 alarms; /* bitvector */
 };
 
 /*
@@ -248,12 +274,6 @@ static ctl_table lm90_dir_table_template[] =
 	{0}
 };
 
-/*
- * Internal variables
- */
-
-static int lm90_id = 0;
-
 /*
  * Real code
  */
@@ -368,11 +388,26 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 			 && (reg_config1 & 0x3F) == 0x00
 			 && reg_convrate <= 0x0A)
 				kind = adm1032;
+			else
+			if (address == 0x4c
+			 && chip_id == 0x51 /* ADT7461 */
+			 && (reg_config1 & 0x1F) == 0x00 /* check compat mode */
+			 && reg_convrate <= 0x0A) 
+				kind = adt7461;
 		}
 		else if (man_id == 0x4D) /* Maxim */
 		{
-			if (address == 0x4C
-			 && (reg_config1 & 0x1F) == 0
+ 			/*
+ 			 * The Maxim variants do NOT have a chip_id register.
+ 			 * Reading from that address will return the last read
+ 			 * value, which in our case is those of the man_id
+ 			 * register. Likewise, the config1 register seems to
+ 			 * lack a low nibble, so the value will be those of the
+ 			 * previous read, so in our case those of the man_id
+ 			 * register.
+ 			 */
+			if (chip_id == man_id
+			 && (reg_config1 & 0x1F) == (man_id & 0x0F)
 			 && reg_convrate <= 0x09)
 				kind = max6657;
 		}
@@ -409,6 +444,11 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 		type_name = "max6657";
 		client_name = "MAX6657 chip";
 	}
+	else if (kind == adt7461)
+	{
+		type_name = "adt7461";
+		client_name = "ADT7561 chip";
+	}
 	else
 	{
 		printk("lm90.o: Unknown kind %d.\n", kind);
@@ -421,8 +461,8 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 	 */
 
 	strcpy(new_client->name, client_name);
-	new_client->id = lm90_id++;
 	data->valid = 0;
+	data->kind = kind;
 	init_MUTEX(&data->update_lock);
 
 	/*
@@ -442,7 +482,7 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 	 */
 
 	if ((err = i2c_register_entry(new_client, type_name,
-	     lm90_dir_table_template)) < 0)
+	     lm90_dir_table_template, THIS_MODULE)) < 0)
 	{
 #ifdef DEBUG
 		printk("lm90.o: Failed registering directory entry.\n");
@@ -594,13 +634,19 @@ static void lm90_local_temp(struct i2c_client *client, int operation,
 	{
 		if (*nrels_mag >= 1)
 		{
-			data->local_high = TEMP1_TO_REG(results[0]);
+			if (data->kind == adt7461)
+				data->local_high = TEMP1_TO_REG_ADT7461(results[0]);
+			else
+				data->local_high = TEMP1_TO_REG(results[0]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_LOCAL_HIGH,
 				data->local_high);
 		}
 		if (*nrels_mag >= 2)
 		{
-			data->local_low = TEMP1_TO_REG(results[1]);
+			if (data->kind == adt7461)
+				data->local_low = TEMP1_TO_REG_ADT7461(results[1]);
+			else
+				data->local_low = TEMP1_TO_REG(results[1]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_LOCAL_LOW,
 				data->local_low);
 		}
@@ -626,7 +672,10 @@ static void lm90_remote_temp(struct i2c_client *client, int operation,
 	{
 		if (*nrels_mag >= 1)
 		{
-			data->remote_high = TEMP2_TO_REG(results[0]);
+			if (data->kind == adt7461)
+				data->remote_high = TEMP2_TO_REG_ADT7461(results[0]);
+			else 
+				data->remote_high = TEMP2_TO_REG(results[0]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_HIGHH,
 				data->remote_high >> 8);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_HIGHL,
@@ -634,7 +683,10 @@ static void lm90_remote_temp(struct i2c_client *client, int operation,
 		}
 		if (*nrels_mag >= 2)
 		{
-			data->remote_low = TEMP2_TO_REG(results[1]);
+			if (data->kind == adt7461)
+				data->remote_low = TEMP2_TO_REG_ADT7461(results[1]);
+			else 
+				data->remote_low = TEMP2_TO_REG(results[1]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_LOWH,
 				data->remote_low >> 8);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_LOWL,
@@ -660,7 +712,10 @@ static void lm90_local_tcrit(struct i2c_client *client, int operation,
 	{
 		if (*nrels_mag >= 1)
 		{
-			data->local_crit = TEMP1_TO_REG(results[0]);
+			if (data->kind == adt7461)
+				data->local_crit = TEMP1_TO_REG_ADT7461(results[0]);
+			else
+				data->local_crit = TEMP1_TO_REG(results[0]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_LOCAL_CRIT,
 				data->local_crit);
 		}
@@ -684,7 +739,10 @@ static void lm90_remote_tcrit(struct i2c_client *client, int operation,
 	{
 		if (*nrels_mag >= 1)
 		{
-			data->remote_crit = TEMP1_TO_REG(results[0]);
+			if (data->kind == adt7461)
+				data->remote_crit = TEMP1_TO_REG_ADT7461(results[0]);
+			else
+				data->remote_crit = TEMP1_TO_REG(results[0]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_CRIT,
 				data->remote_crit);
 		}
