@@ -239,7 +239,6 @@ enum sensor_type { fan, temp, vid, pll, lcd, alarm };
    SMBus and not on the ISA bus. */
 struct maxi_data {
 	struct i2c_client client;
-	struct semaphore lock;
 	int sysctl_id;
 	enum maxi_type type;
 
@@ -306,7 +305,6 @@ static void maxi_alarms(struct i2c_client *client, int operation,
 /* The driver. I choose to use type i2c_driver, as at is identical to
    the smbus_driver. */
 static struct i2c_driver maxi_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "HP MaxiLife driver",
 	.id		= I2C_DRIVERID_MAXILIFE,
 	.flags		= I2C_DF_NOTIFY,
@@ -314,8 +312,6 @@ static struct i2c_driver maxi_driver = {
 	.detach_client	= maxi_detach_client,
 };
 
-static int maxi_id = 0;
-
 /* Default firmware version. Use module option "maxi_version"
    to set desired version. Auto detect is not yet working */
 static int maxi_version = cristal;
@@ -603,10 +599,7 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 			    ((struct maxi_data *) (new_client->data))->
 			    lcd[i][j] = (u8) 0;
 
-	new_client->id = maxi_id++;
-
 	data->valid = 0;
-	init_MUTEX(&data->lock);
 	init_MUTEX(&data->update_lock);
 
 	/* Tell i2c-core that a new client has arrived */
@@ -615,7 +608,8 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((err = i2c_register_entry(new_client, type_name,
-					  maxi_dir_table_template)) < 0)
+					  maxi_dir_table_template,
+					  THIS_MODULE)) < 0)
 		goto ERROR4;
 	data->sysctl_id = err;
 
