@@ -252,11 +252,8 @@ static void mtp008_sens(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 static void mtp008_getsensortype(struct mtp008_data *data, u8 inp);
 
-static int mtp008_id = 0;
-
 static struct i2c_driver mtp008_driver =
 {
-	.owner		= THIS_MODULE,
 	.name		= "MTP008 sensor driver",
 	.id		= I2C_DRIVERID_MTP008,
 	.flags		= I2C_DF_NOTIFY,
@@ -429,8 +426,6 @@ int mtp008_detect(struct i2c_adapter *adapter, int address,
 	client_name = "MTP008 chip";
 	strcpy(new_client->name, client_name);
 	data->type = kind;
-
-	new_client->id = mtp008_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -444,7 +439,8 @@ int mtp008_detect(struct i2c_adapter *adapter, int address,
 	 * Register a new directory entry with the sensors module.
 	 */
 	if ((sysid = i2c_register_entry(new_client, type_name,
-					    mtp008_dir_table_template)) < 0) {
+					    mtp008_dir_table_template,
+					    THIS_MODULE)) < 0) {
 		err = sysid;
 		goto ERROR2;
 	}
