@@ -41,6 +41,7 @@
 #include <linux/init.h>
 #include <asm/io.h>
 #include "version.h"
+#include "sensors_vid.h"
 
 static unsigned short normal_i2c[] = { SENSORS_I2C_END };
 static unsigned short normal_i2c_range[] = { SENSORS_I2C_END };
@@ -49,7 +50,7 @@ static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
 static struct i2c_force_data forces[] = {{NULL}};
 static u8 devid;
 static unsigned int extra_isa[] = { 0x0000, 0x0000, 0x0000 };
-static u8 confreg[3];
+static u8 confreg[4];
 
 enum chips { any_chip, pc87360, pc87363, pc87364, pc87365, pc87366 };
 static struct i2c_address_data addr_data = {
@@ -77,9 +78,6 @@ MODULE_PARM_DESC(init,
  * Super-I/O registers and operations
  */
 
-#define REG	0x2e	/* The register to read/write */
-#define VAL	0x2f	/* The value to read/write */
-
 #define DEV	0x07	/* Register: Logical device select */
 #define DEVID	0x20	/* Register: Device ID */
 #define ACT	0x30	/* Register: Device activation */
@@ -94,22 +92,22 @@ static const u8 logdev[3] = { FSCM, VLM, TMS };
 #define LD_IN		1
 #define LD_TEMP		2
 
-static inline void superio_outb(int reg, int val)
+static inline void superio_outb(int sioaddr, int reg, int val)
 {
-	outb(reg, REG);
-	outb(val, VAL);
+	outb(reg, sioaddr);
+	outb(val, sioaddr+1);
 }
 
-static inline int superio_inb(int reg)
+static inline int superio_inb(int sioaddr, int reg)
 {
-	outb(reg, REG);
-	return inb(VAL);
+	outb(reg, sioaddr);
+	return inb(sioaddr+1);
 }
 
-static inline void superio_exit(void)
+static inline void superio_exit(int sioaddr)
 {
-	outb(0x02, REG);
-	outb(0x02, VAL);
+	outb(0x02, sioaddr);
+	outb(0x02, sioaddr+1);
 }
 
 /*
@@ -168,6 +166,7 @@ static inline u8 PWM_TO_REG(int val, int inv)
 #define PC87365_REG_IN_STATUS		0x0A
 #define PC87365_REG_IN_ALARMS1		0x00
 #define PC87365_REG_IN_ALARMS2		0x01
+#define PC87365_REG_VID			0x06
 
 #define IN_FROM_REG(val,ref)		(((val) * (ref) + 128) / 256)
 #define IN_TO_REG(val,ref)		((val)<0 ? 0 : \
@@ -215,6 +214,9 @@ struct pc87360_data {
 	u8 in_crit[3];		/* Register value */
 	u8 in_status[14];	/* Register value */
 	u16 in_alarms;		/* Register values, combined, masked */
+	u8 vid_conf;		/* Configuration register value */
+	u8 vrm;
+	u8 vid;			/* Register value */
 
 	u8 temp[3];		/* Register value */
 	u8 temp_min[3];		/* Register value */
@@ -236,7 +238,6 @@ static void pc87360_write_value(struct pc87360_data *data, u8 ldi, u8 bank,
 				u8 reg, u8 value);
 static void pc87360_init_client(struct i2c_client *client, int use_thermistors);
 static void pc87360_update_client(struct i2c_client *client);
-static int pc87360_find(u8 *devid, int *address);
 
 
 void pc87365_alarms(struct i2c_client *client, int operation, int ctl_name,
@@ -255,16 +256,17 @@ void pc87365_in(struct i2c_client *client, int operation, int ctl_name,
 		int *nrels_mag, long *results);
 void pc87365_in_status(struct i2c_client *client, int operation, int ctl_name,
 		       int *nrels_mag, long *results);
+void pc87365_vid(struct i2c_client *client, int operation, int ctl_name,
+		 int *nrels_mag, long *results);
+void pc87365_vrm(struct i2c_client *client, int operation, int ctl_name,
+		 int *nrels_mag, long *results);
 
 void pc87365_temp(struct i2c_client *client, int operation, int ctl_name,
 		  int *nrels_mag, long *results);
 void pc87365_temp_status(struct i2c_client *client, int operation, int ctl_name,
 			 int *nrels_mag, long *results);
 
-static int pc87360_id = 0;
-
 static struct i2c_driver pc87360_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "PC8736x hardware monitor",
 	.flags		= I2C_DF_NOTIFY,
 	.attach_adapter	= pc87360_attach_adapter,
@@ -319,6 +321,9 @@ static struct i2c_driver pc87360_driver = {
 #define PC87365_SYSCTL_TEMP5_STATUS	2312 /* not for PC87365 */
 #define PC87365_SYSCTL_TEMP6_STATUS	2313 /* not for PC87365 */
 
+#define PC87365_SYSCTL_VID		2400
+#define PC87365_SYSCTL_VRM		2401
+
 #define PC87365_STATUS_IN_MIN		0x02
 #define PC87365_STATUS_IN_MAX		0x04
 
@@ -451,6 +456,10 @@ static ctl_table pc87365_dir_table_template[] = { /* PC87366 too */
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &pc87365_in_status},
 	{PC87365_SYSCTL_TEMP6_STATUS, "temp6_status", NULL, 0, 0444, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &pc87365_in_status},
+	{PC87365_SYSCTL_VID, "vid", NULL, 0, 0444, NULL,
+	 &i2c_proc_real, &i2c_sysctl_real, NULL, &pc87365_vid},
+	{PC87365_SYSCTL_VRM, "vrm", NULL, 0, 0644, NULL,
+	 &i2c_proc_real, &i2c_sysctl_real, NULL, &pc87365_vrm},
 	{0}
 };
 
@@ -459,7 +468,7 @@ static int pc87360_attach_adapter(struct i2c_adapter *adapter)
 	return i2c_detect(adapter, &addr_data, pc87360_detect);
 }
 
-static int pc87360_find(u8 *devid, int *address)
+static int pc87360_find(int sioaddr, u8 *devid, int *address)
 {
 	u16 val;
 	int i;
@@ -468,7 +477,7 @@ static int pc87360_find(u8 *devid, int *address)
 	/* No superio_enter */
 
 	/* Identify device */
-	val = superio_inb(DEVID);
+	val = superio_inb(sioaddr, DEVID);
 	switch (val) {
 	case 0xE1: /* PC87360 */
 	case 0xE8: /* PC87363 */
@@ -480,7 +489,7 @@ static int pc87360_find(u8 *devid, int *address)
 		nrdev = 3;
 		break;
 	default:
-		superio_exit();
+		superio_exit(sioaddr);
 		return -ENODEV;
 	}
 	/* Remember the device id */
@@ -488,19 +497,19 @@ static int pc87360_find(u8 *devid, int *address)
 
 	for (i = 0; i < nrdev; i++) {
 		/* select logical device */
-		superio_outb(DEV, logdev[i]);
+		superio_outb(sioaddr, DEV, logdev[i]);
 
-		val = superio_inb(ACT);
+		val = superio_inb(sioaddr, ACT);
 		if (!(val & 0x01)) {
-			printk(KERN_INFO "pc87360.o: device 0x%02x not "
+			printk(KERN_INFO "pc87360.o: Device 0x%02x not "
 			       "activated\n", logdev[i]);
 			continue;
 		}
 
-		val = (superio_inb(BASE) << 8)
-		    | superio_inb(BASE + 1);
+		val = (superio_inb(sioaddr, BASE) << 8)
+		    | superio_inb(sioaddr, BASE + 1);
 		if (!val) {
-			printk(KERN_INFO "pc87360.o: base address not set for "
+			printk(KERN_INFO "pc87360.o: Base address not set for "
 			       "device 0x%02x\n", logdev[i]);
 			continue;
 		}
@@ -508,8 +517,8 @@ static int pc87360_find(u8 *devid, int *address)
 		address[i] = val;
 
 		if (i==0) { /* Fans */
-			confreg[0] = superio_inb(0xF0);
-			confreg[1] = superio_inb(0xF1);
+			confreg[0] = superio_inb(sioaddr, 0xF0);
+			confreg[1] = superio_inb(sioaddr, 0xF1);
 			
 #ifdef DEBUG
 			printk(KERN_DEBUG "pc87360.o: Fan 1: mon=%d "
@@ -525,21 +534,27 @@ static int pc87360_find(u8 *devid, int *address)
 		} else if (i==1) { /* Voltages */
 			/* Are we using thermistors? */
 			if (*devid == 0xE9) { /* PC87366 */
-				/* This register is not logical-device
-				   specific, just that we won't need it if we
-				   don't use the VLM device */
-				confreg[2] = superio_inb(0x2B);
+				/* These registers are not logical-device
+				   specific, just that we won't need them if
+				   we don't use the VLM device */
+				confreg[2] = superio_inb(sioaddr, 0x2B);
+				confreg[3] = superio_inb(sioaddr, 0x25);
 
 				if (confreg[2] & 0x40) {
 					printk(KERN_INFO "pc87360.o: Using "
 					       "thermistors for temperature "
 					       "monitoring\n");
 				}
+				if (confreg[3] & 0xE0) {
+					printk(KERN_INFO "pc87360.o: VID "
+					       "inputs routed (mode %u)\n",
+					       	confreg[3] >> 5);
+				}
 			}
 		}
 	}
 
-	superio_exit();
+	superio_exit(sioaddr);
 	return 0;
 }
 
@@ -564,8 +579,9 @@ int pc87360_detect(struct i2c_adapter *adapter, int address,
 	for (i = 0; i < 3; i++) {
 		if (extra_isa[i]
 		 && check_region(extra_isa[i], PC87360_EXTENT)) {
-			printk(KERN_ERR "pc87360.o: region 0x%x already in "
-			       "use!\n", address);
+			printk(KERN_ERR "pc87360.o: Region 0x%x-0x%x already "
+			       "in use!\n", extra_isa[i],
+			       extra_isa[i]+PC87360_EXTENT-1);
 			return -ENODEV;
 		}
 	}
@@ -621,8 +637,6 @@ int pc87360_detect(struct i2c_adapter *adapter, int address,
 		}
 	}
 	strcpy(new_client->name, client_name);
-
-	new_client->id = pc87360_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -639,17 +653,22 @@ int pc87360_detect(struct i2c_adapter *adapter, int address,
 		 	i &= pc87360_read_value(data, LD_TEMP, NO_BANK,
 						PC87365_REG_TEMP_CONFIG);
 		}
-		data->in_vref = (i&0x02) ? 303 : 297;
+		data->in_vref = (i&0x02) ? 3025 : 2966;
 #ifdef DEBUG
-		printk(KERN_DEBUG "Using %s reference voltage\n",
+		printk(KERN_DEBUG "pc87360.o: Using %s reference voltage\n",
 		       (i&0x02) ? "external" : "internal");
 #endif
+
+		data->vid_conf = confreg[3];
+		data->vrm = 90;
 	}
 
 	/* Fan clock dividers may be needed before any data is read */
 	for (i = 0; i < data->fannr; i++) {
-		data->fan_status[i] = pc87360_read_value(data, LD_FAN,
-				      NO_BANK, PC87360_REG_FAN_STATUS(i));
+		if (FAN_CONFIG_MONITOR(data->fan_conf, i))
+			data->fan_status[i] = pc87360_read_value(data,
+					      LD_FAN, NO_BANK,
+					      PC87360_REG_FAN_STATUS(i));
 	}
 
 	if (init > 0) {
@@ -660,7 +679,7 @@ int pc87360_detect(struct i2c_adapter *adapter, int address,
 	}
 
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
-				    type_name, template)) < 0) {
+				    type_name, template, THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR2;
 	}
@@ -738,18 +757,14 @@ static void pc87360_init_client(struct i2c_client *client, int use_thermistors)
 	const u8 init_temp[3] = { 2, 2, 1 };
 	u8 reg;
 
-	if (init >= 3 && data->innr) {
+	if (init >= 2 && data->innr) {
 		reg = pc87360_read_value(data, LD_IN, NO_BANK,
 					 PC87365_REG_IN_CONVRATE);
-		if (reg & 0x06) {
-#ifdef DEBUG
-			printk(KERN_DEBUG "pc87360.o: Setting "
-			       "VLM conversion period to 1 second\n");
-#endif		
-			pc87360_write_value(data, LD_IN, NO_BANK,
-					    PC87365_REG_IN_CONVRATE,
-					    (reg & 0xF8) | 0x01);
-		}
+		printk(KERN_INFO "pc87360.o: VLM conversion set to"
+		       "1s period, 160us delay\n");
+		pc87360_write_value(data, LD_IN, NO_BANK,
+				    PC87365_REG_IN_CONVRATE,
+				    (reg & 0xC0) | 0x11);
 	}
 
 	nr = data->innr < 11 ? data->innr : 11;
@@ -803,8 +818,14 @@ static void pc87360_init_client(struct i2c_client *client, int use_thermistors)
 				   diodes */
 				reg = pc87360_read_value(data, LD_TEMP, (i-11)/2,
 							 PC87365_REG_TEMP_STATUS);
-				if (reg & 0x01)
+				if (reg & 0x01) {
+#ifdef DEBUG
+					printk(KERN_DEBUG "pc87360.o: Skipping "
+					       "temp%d, pin already in use by "
+					       "temp%d\n", i-7, (i-11)/2);
+#endif
 					continue;
+				}
 			
 				/* Forcibly enable thermistor channel */
 				reg = pc87360_read_value(data, LD_IN, i,
@@ -848,6 +869,19 @@ static void pc87360_init_client(struct i2c_client *client, int use_thermistors)
 					    PC87365_REG_TEMP_CONFIG,
 					    reg & 0xFE);
 		}
+
+		if (init >= 2) {
+			/* Chip config as documented by National Semi. */
+			pc87360_write_value(data, LD_TEMP, 0xF, 0xA, 0x08);
+			/* We voluntarily omit the bank here, in case the
+			   sequence itself matters. It shouldn't be a problem,
+			   since nobody else is supposed to access the
+			   device at that point. */
+			pc87360_write_value(data, LD_TEMP, NO_BANK, 0xB, 0x04);
+			pc87360_write_value(data, LD_TEMP, NO_BANK, 0xC, 0x35);
+			pc87360_write_value(data, LD_TEMP, NO_BANK, 0xD, 0x05);
+			pc87360_write_value(data, LD_TEMP, NO_BANK, 0xE, 0x05);
+		}
 	}
 }
 
@@ -960,6 +994,9 @@ static void pc87360_update_client(struct i2c_client *client)
 					| ((pc87360_read_value(data, LD_IN,
 					    NO_BANK, PC87365_REG_IN_ALARMS2)
 					    & 0x07) << 8);
+			data->vid = (data->vid_conf & 0xE0) ?
+				    pc87360_read_value(data, LD_IN,
+				    NO_BANK, PC87365_REG_VID) : 0x1F;
 		}
 
 		/* Temperatures */
@@ -1072,7 +1109,10 @@ void pc87360_fan_div(struct i2c_client *client, int operation,
 		for (i = 0; i < data->fannr; i++) {
 			results[i] = FAN_DIV_FROM_REG(data->fan_status[i]);
 		}
-		*nrels_mag = data->fannr;
+		for (; i < 3; i++) {
+			results[i] = 0;
+		}
+		*nrels_mag = 3;
 	}
 }
 
@@ -1126,7 +1166,7 @@ void pc87365_in(struct i2c_client *client, int operation, int ctl_name,
 	int nr = ctl_name - PC87365_SYSCTL_IN0;
 
 	if (operation == SENSORS_PROC_REAL_INFO)
-		*nrels_mag = 2;
+		*nrels_mag = 3;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		pc87360_update_client(client);
 		results[0] = IN_FROM_REG(data->in_min[nr], data->in_vref);
@@ -1181,6 +1221,34 @@ void pc87365_in_status(struct i2c_client *client, int operation, int ctl_name,
 	}
 }
 
+void pc87365_vid(struct i2c_client *client, int operation, int ctl_name,
+		 int *nrels_mag, long *results)
+{
+	struct pc87360_data *data = client->data;
+	if (operation == SENSORS_PROC_REAL_INFO)
+		*nrels_mag = 3;
+	else if (operation == SENSORS_PROC_REAL_READ) {
+		pc87360_update_client(client);
+		results[0] = vid_from_reg(data->vid & 0x1f, data->vrm);
+		*nrels_mag = 1;
+	}
+}
+
+void pc87365_vrm(struct i2c_client *client, int operation, int ctl_name,
+		 int *nrels_mag, long *results)
+{
+	struct pc87360_data *data = client->data;
+	if (operation == SENSORS_PROC_REAL_INFO)
+		*nrels_mag = 1;
+	else if (operation == SENSORS_PROC_REAL_READ) {
+		results[0] = data->vrm;
+		*nrels_mag = 1;
+	} else if (operation == SENSORS_PROC_REAL_WRITE) {
+		if (*nrels_mag >= 1)
+			data->vrm = results[0];
+	}
+}
+
 void pc87365_temp(struct i2c_client *client, int operation, int ctl_name,
 		  int *nrels_mag, long *results)
 {
@@ -1243,7 +1311,8 @@ static int __init pc87360_init(void)
 
 	printk(KERN_INFO "pc87360.o version %s (%s)\n", LM_VERSION, LM_DATE);
 
-	if (pc87360_find(&devid, extra_isa)) {
+	if (pc87360_find(0x2e, &devid, extra_isa)
+	 && pc87360_find(0x4e, &devid, extra_isa)) {
 		printk(KERN_WARNING "pc87360.o: PC8736x not detected, "
 		       "module not inserted.\n");
 		return -ENODEV;
