@@ -102,7 +102,6 @@ static void pcf8591_aout(struct i2c_client *client, int operation,
 
 /* This is the driver that will be inserted */
 static struct i2c_driver pcf8591_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "PCF8591 sensor chip driver",
 	.id		= I2C_DRIVERID_PCF8591,
 	.flags		= I2C_DF_NOTIFY,
@@ -110,8 +109,6 @@ static struct i2c_driver pcf8591_driver = {
 	.detach_client	= pcf8591_detach_client,
 };
 
-static int pcf8591_id = 0;
-
 /* The /proc/sys entries */
 
 /* -- SENSORS SYSCTL START -- */
@@ -208,8 +205,6 @@ int pcf8591_detect(struct i2c_adapter *adapter, int address,
 
         /* Fill in the remaining client fields and put it into the global list */
         strcpy(new_client->name, client_name);
-
-        new_client->id = pcf8591_id++;
         data->valid = 0;
         init_MUTEX(&data->update_lock);
 
@@ -220,7 +215,8 @@ int pcf8591_detect(struct i2c_adapter *adapter, int address,
         /* Register a new directory entry with module sensors */
         if ((i = i2c_register_entry(new_client,
                                         type_name,
-                                        pcf8591_dir_table_template)) < 0) {
+                                        pcf8591_dir_table_template,
+					THIS_MODULE)) < 0) {
                 err = i;
                 goto ERROR4;
         }
