@@ -112,7 +112,6 @@ static void thmc50_update_client(struct i2c_client *client);
 
 /* This is the driver that will be inserted */
 static struct i2c_driver thmc50_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "THMC50 sensor chip driver",
 	.id		= I2C_DRIVERID_THMC50,
 	.flags		= I2C_DF_NOTIFY,
@@ -153,8 +152,6 @@ static ctl_table thmc50_dir_table_template[] = {
 };
 
 
-static int thmc50_id = 0;
-
 static int thmc50_attach_adapter(struct i2c_adapter *adapter)
 {
 	return i2c_detect(adapter, &addr_data, thmc50_detect);
@@ -232,8 +229,6 @@ int thmc50_detect(struct i2c_adapter *adapter, int address,
 
 	/* Fill in the remaining client fields and put it into the global list */
 	strcpy(new_client->name, client_name);
-
-	new_client->id = thmc50_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -243,7 +238,8 @@ int thmc50_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					thmc50_dir_table_template)) < 0) {
+					thmc50_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
