@@ -40,6 +40,7 @@
 #include <linux/init.h>
 #include <asm/io.h>
 #include "version.h"
+#include "sensors_compat.h"
 
 
 /* If force_addr is set to anything different from 0, we forcibly enable
@@ -356,12 +357,9 @@ static void via686a_alarms(struct i2c_client *client, int operation,
 static void via686a_fan_div(struct i2c_client *client, int operation,
 			    int ctl_name, int *nrels_mag, long *results);
 
-static int via686a_id = 0;
-
 /* The driver. I choose to use type i2c_driver, as at is identical to both
    smbus_driver and isa_driver, and clients could be of either kind */
 static struct i2c_driver via686a_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "VIA 686A",
 	.id		= I2C_DRIVERID_VIA686A,
 	.flags		= I2C_DF_NOTIFY,
@@ -512,8 +510,6 @@ int via686a_detect(struct i2c_adapter *adapter, int address,
 
 	/* Fill in the remaining client fields and put into the global list */
 	strcpy(new_client->name, "Via 686A Integrated Sensors");
-
-	new_client->id = via686a_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -524,7 +520,8 @@ int via686a_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					via686a_dir_table_template)) < 0) {
+					via686a_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
