@@ -232,10 +232,7 @@ static void vt1211_uch(struct i2c_client *client, int operation,
 static void vt1211_temp(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 
-static int vt1211_id = 0;
-
 static struct i2c_driver vt1211_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "VT1211 sensors driver",
 	.id		= I2C_DRIVERID_VT1211,
 	.flags		= I2C_DF_NOTIFY,
@@ -424,8 +421,6 @@ int vt1211_detect(struct i2c_adapter *adapter, int address,
 
 	request_region(address, VT1211_EXTENT, "vt1211-sensors");
 	strcpy(new_client->name, client_name);
-
-	new_client->id = vt1211_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -434,7 +429,8 @@ int vt1211_detect(struct i2c_adapter *adapter, int address,
 
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					vt1211_dir_table_template)) < 0) {
+					vt1211_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
