@@ -200,10 +200,6 @@ static inline u8 FAN_TO_REG(long rpm, int div)
                                                  ((val)+5)/10),0,255))
 #define TEMP_FROM_REG(val) (((val)>0x80?(val)-0x100:(val))*10)
 
-#define AS99127_TEMP_ADD_TO_REG(val) (SENSORS_LIMIT((((((val) + 2)*4)/10) \
-                                               << 7),0,0xffff))
-#define AS99127_TEMP_ADD_FROM_REG(val) ((((val) >> 7) * 10) / 4)
-
 #define ALARMS_FROM_REG(val) (val)
 #define PWM_FROM_REG(val) (val)
 #define PWM_TO_REG(val) (SENSORS_LIMIT((val),0,255))
@@ -329,10 +325,7 @@ static void w83781d_rt(struct i2c_client *client, int operation,
 		       int ctl_name, int *nrels_mag, long *results);
 #endif
 
-static int w83781d_id = 0;
-
 static struct i2c_driver w83781d_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "W83781D sensor driver",
 	.id		= I2C_DRIVERID_W83781D,
 	.flags		= I2C_DF_NOTIFY,
@@ -841,28 +834,24 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 	   bank. */
 	if (kind < 0) {
 		if (w83781d_read_value(new_client, W83781D_REG_CONFIG) &
-		    0x80) {
-			err = -ENODEV;
+		    0x80)
 			goto ERROR1;
-		}
+
 		val1 = w83781d_read_value(new_client, W83781D_REG_BANK);
 		val2 = w83781d_read_value(new_client, W83781D_REG_CHIPMAN);
 		/* Check for Winbond or Asus ID if in bank 0 */
 		if ((!(val1 & 0x07)) &&
 		    (((!(val1 & 0x80)) && (val2 != 0xa3) && (val2 != 0xc3))
-		     || ((val1 & 0x80) && (val2 != 0x5c) && (val2 != 0x12)))) {
-			err = -ENODEV;
+		     || ((val1 & 0x80) && (val2 != 0x5c) && (val2 != 0x12))))
 			goto ERROR1;
-		}
+
 		/* If Winbond SMBus, check address at 0x48.
 		   Asus doesn't support, except for the as99127f rev.2 */
 		if ((!is_isa) && (((!(val1 & 0x80)) && (val2 == 0xa3)) ||
 				  ((val1 & 0x80) && (val2 == 0x5c)))) {
 			if (w83781d_read_value
-			    (new_client, W83781D_REG_I2C_ADDR) != address) {
-				err = -ENODEV;
+			    (new_client, W83781D_REG_I2C_ADDR) != address)
 				goto ERROR1;
-			}
 		}
 	}
 
@@ -881,10 +870,9 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 			vendid = winbond;
 		else if (val2 == 0x12)
 			vendid = asus;
-		else {
-			err = -ENODEV;
+		else
 			goto ERROR1;
-		}
+
 		val1 =
 		    w83781d_read_value(new_client, W83781D_REG_WCHIPID);
 		if ((val1 == 0x10 || val1 == 0x11) && vendid == winbond)
@@ -907,7 +895,6 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 				    (KERN_WARNING "w83781d.o: Ignoring 'force' parameter for unknown chip at"
 				     "adapter %d, address 0x%02x\n",
 				     i2c_adapter_id(adapter), address);
-			err = -EINVAL;
 			goto ERROR1;
 		}
 	}
@@ -949,8 +936,6 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 	/* Fill in the remaining client fields and put it into the global list */
 	strcpy(new_client->name, client_name);
 	data->type = kind;
-
-	new_client->id = w83781d_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -972,7 +957,6 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 				   force_subclients[i] > 0x4f) {
 					printk(KERN_ERR "w83781d.o: Invalid subclient address %d; must be 0x48-0x4f\n",
 					        force_subclients[i]);
-					err = -EINVAL;
 					goto ERROR5;
 				}
 			}
@@ -996,7 +980,6 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 			if(data->lm75[0].addr == data->lm75[1].addr) {
 				printk(KERN_ERR "w83781d.o: Duplicate addresses 0x%x for subclients.\n",
 					data->lm75[0].addr);
-				err = -EBUSY;
 				goto ERROR5;
 			}
 		}
@@ -1020,7 +1003,6 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 			data->lm75[i].driver = &w83781d_driver;
 			data->lm75[i].flags = 0;
 			strcpy(data->lm75[i].name, client_name);
-			data->lm75[i].id = w83781d_id++;
 			if ((err = i2c_attach_client(&(data->lm75[i])))) {
 				printk(KERN_ERR "w83781d.o: Subclient %d registration at address 0x%x failed.\n",
 				       i, data->lm75[i].addr);
@@ -1050,7 +1032,8 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
                                             w83791d_dir_table_template :
                                         (is_isa || kind == w83627hf) ?
 					   w83782d_isa_dir_table_template :
-					   w83782d_i2c_dir_table_template)) < 0) {
+					   w83782d_i2c_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR7;
 	}
@@ -1264,7 +1247,7 @@ static int w83781d_write_value(struct i2c_client *client, u16 reg, u16 value)
 static void w83781d_init_client(struct i2c_client *client)
 {
 	struct w83781d_data *data = client->data;
-	int vid = 0, i, p;
+	int i, p;
 	int type = data->type;
 	u8 tmp;
 
@@ -1286,17 +1269,7 @@ static void w83781d_init_client(struct i2c_client *client)
 		w83781d_write_value(client, W83781D_REG_BEEP_INTS2, 0);
 	}
 
-	if (type != w83697hf) {
-		vid = w83781d_read_value(client, W83781D_REG_VID_FANDIV) & 0x0f;
-		vid |=
-		    (w83781d_read_value(client, W83781D_REG_CHIPID) & 0x01) << 4;
-                if (type == w83791d) {
-                        data->vrm = 92;
-                } else {
-		data->vrm = DEFAULT_VRM;
-                }
-		vid = vid_from_reg(vid, data->vrm);
-	}
+	data->vrm = (type == w83791d) ? 90 : 82;
 
 	if ((type != w83781d) && (type != as99127f)) {
 		tmp = w83781d_read_value(client, W83781D_REG_SCFG1);
@@ -1344,7 +1317,7 @@ static void w83781d_init_client(struct i2c_client *client)
 	}
 #endif				/* W83781D_RT */
 
-	if(init) {
+	if (init && type != as99127f) {
 		w83781d_write_value(client, W83781D_REG_TEMP2_CONFIG, 0x00);
 		if (type != w83783s && type != w83697hf) {
 			w83781d_write_value(client, W83781D_REG_TEMP3_CONFIG,
@@ -1606,43 +1579,22 @@ void w83781d_temp_add(struct i2c_client *client, int operation,
 		*nrels_mag = 1;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		w83781d_update_client(client);
-		if (data->type == as99127f) {
-			results[0] =
-			    AS99127_TEMP_ADD_FROM_REG(data->
-						      temp_add_over[nr]);
-			results[1] =
-			    AS99127_TEMP_ADD_FROM_REG(data->
-						      temp_add_hyst[nr]);
-			results[2] =
-			    AS99127_TEMP_ADD_FROM_REG(data->temp_add[nr]);
-		} else {
-			results[0] =
-			    LM75_TEMP_FROM_REG(data->temp_add_over[nr]);
-			results[1] =
-			    LM75_TEMP_FROM_REG(data->temp_add_hyst[nr]);
-			results[2] = LM75_TEMP_FROM_REG(data->temp_add[nr]);
-		}
+		results[0] = LM75_TEMP_FROM_REG(data->temp_add_over[nr]);
+		results[1] = LM75_TEMP_FROM_REG(data->temp_add_hyst[nr]);
+		results[2] = LM75_TEMP_FROM_REG(data->temp_add[nr]);
 		*nrels_mag = 3;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			if (data->type == as99127f)
-				data->temp_add_over[nr] =
-				    AS99127_TEMP_ADD_TO_REG(results[0]);
-			else
-				data->temp_add_over[nr] =
-				    LM75_TEMP_TO_REG(results[0]);
+			data->temp_add_over[nr] =
+			    LM75_TEMP_TO_REG(results[0]);
 			w83781d_write_value(client,
 					    nr ? W83781D_REG_TEMP3_OVER :
 					    W83781D_REG_TEMP2_OVER,
 					    data->temp_add_over[nr]);
 		}
 		if (*nrels_mag >= 2) {
-			if (data->type == as99127f)
-				data->temp_add_hyst[nr] =
-				    AS99127_TEMP_ADD_TO_REG(results[1]);
-			else
-				data->temp_add_hyst[nr] =
-				    LM75_TEMP_TO_REG(results[1]);
+			data->temp_add_hyst[nr] =
+			    LM75_TEMP_TO_REG(results[1]);
 			w83781d_write_value(client,
 					    nr ? W83781D_REG_TEMP3_HYST :
 					    W83781D_REG_TEMP2_HYST,
