@@ -372,6 +372,7 @@
 #define SENSORS_ADM1027_PREFIX "adm1027"
 #define SENSORS_ADT7463_PREFIX "adt7463"
 #define SENSORS_EMC6D100_PREFIX "emc6d100"
+#define SENSORS_EMC6D102_PREFIX "emc6d102"
 
 #define SENSORS_ADM1027_ALARM_MASK           1  /* RW -- alarm_mask  */
 #define SENSORS_ADM1027_FAN1_PPR             2  /* RW -- fan1_ppr  */
@@ -475,13 +476,14 @@
 #define SENSORS_LM85_ZONE3_RANGE           100  /* RW -- zone3_range  */
 #define SENSORS_LM85_ZONE3_SMOOTH          101  /* RW -- zone3_smooth  */
 
-/* LM86/LM89/LM90/LM99/ADM1032/MAX6657 chips */
+/* LM86/LM89/LM90/LM99/ADM1032/MAX6657/ADT7461 chips */
 
 #define SENSORS_LM90_PREFIX "lm90"
 #define SENSORS_ADM1032_PREFIX "adm1032"
 #define SENSORS_LM99_PREFIX "lm99"
 #define SENSORS_LM86_PREFIX "lm86"
 #define SENSORS_MAX6657_PREFIX "max6657"
+#define SENSORS_ADT7461_PREFIX "adt7461"
 
 #define SENSORS_LM90_LOCAL_TEMP 51 /* R */
 #define SENSORS_LM90_LOCAL_HIGH 52 /* RW */
@@ -495,6 +497,23 @@
 #define SENSORS_LM90_REMOTE_TCRIT_HYST 80 /* R, see driver source */
 #define SENSORS_LM90_ALARMS 81 /* R */
 
+/* LM63 chips */
+
+#define SENSORS_LM63_PREFIX "lm63"
+
+#define SENSORS_LM63_LOCAL_TEMP			51 /* R  */
+#define SENSORS_LM63_LOCAL_HIGH			52 /* RW */
+#define SENSORS_LM63_REMOTE_TEMP		57 /* R  */
+#define SENSORS_LM63_REMOTE_HIGH		58 /* RW */
+#define SENSORS_LM63_REMOTE_LOW			59 /* RW */
+#define SENSORS_LM63_REMOTE_TCRIT		60 /* R  */
+#define SENSORS_LM63_REMOTE_TCRIT_HYST		80 /* RW */
+#define SENSORS_LM63_ALARMS			81 /* R  */
+#define SENSORS_LM63_FAN			84 /* R  */
+#define SENSORS_LM63_FAN_LOW			85 /* RW */
+#define SENSORS_LM63_PWM			87 /* RW */
+#define SENSORS_LM63_PWM_ENABLE			88 /* RW */
+
 /* Winbond W83781D chips */
 
 #define SENSORS_W83781D_PREFIX "w83781d"
@@ -632,6 +651,81 @@
 
 
 
+/* Winbond W83792AD/D chip */
+#define SENSORS_W83792D_PREFIX "w83792d"
+
+
+#define SENSORS_W83792D_IN0 1 /* R */
+#define SENSORS_W83792D_IN1 2 /* R */
+#define SENSORS_W83792D_IN2 3 /* R */
+#define SENSORS_W83792D_IN3 4 /* R */
+#define SENSORS_W83792D_IN4 5 /* R */
+#define SENSORS_W83792D_IN5 6 /* R */
+#define SENSORS_W83792D_IN6 7 /* R */
+#define SENSORS_W83792D_IN7 8 /* R */
+#define SENSORS_W83792D_IN8 9 /* R */
+#define SENSORS_W83792D_IN9 10 /* R */
+
+#define SENSORS_W83792D_IN0_MIN 11 /* RW */
+#define SENSORS_W83792D_IN1_MIN 12 /* RW */
+#define SENSORS_W83792D_IN2_MIN 13 /* RW */
+#define SENSORS_W83792D_IN3_MIN 14 /* RW */
+#define SENSORS_W83792D_IN4_MIN 15 /* RW */
+#define SENSORS_W83792D_IN5_MIN 16 /* RW */
+#define SENSORS_W83792D_IN6_MIN 17 /* RW */
+#define SENSORS_W83792D_IN7_MIN 18 /* RW */
+#define SENSORS_W83792D_IN8_MIN 19 /* RW */
+#define SENSORS_W83792D_IN9_MIN 20 /* RW */
+
+#define SENSORS_W83792D_IN0_MAX 21 /* RW */
+#define SENSORS_W83792D_IN1_MAX 22 /* RW */
+#define SENSORS_W83792D_IN2_MAX 23 /* RW */
+#define SENSORS_W83792D_IN3_MAX 24 /* RW */
+#define SENSORS_W83792D_IN4_MAX 25 /* RW */
+#define SENSORS_W83792D_IN5_MAX 26 /* RW */
+#define SENSORS_W83792D_IN6_MAX 27 /* RW */
+#define SENSORS_W83792D_IN7_MAX 28 /* RW */
+#define SENSORS_W83792D_IN8_MAX 29 /* RW */
+#define SENSORS_W83792D_IN9_MAX 30 /* RW */
+
+#define SENSORS_W83792D_FAN1 31 /* R */
+#define SENSORS_W83792D_FAN2 32 /* R */
+#define SENSORS_W83792D_FAN3 33 /* R */
+#define SENSORS_W83792D_FAN4 34 /* R */
+#define SENSORS_W83792D_FAN5 35 /* R */
+#define SENSORS_W83792D_FAN6 36 /* R */
+#define SENSORS_W83792D_FAN7 37 /* R */
+
+#define SENSORS_W83792D_FAN1_MIN 41 /* RW */
+#define SENSORS_W83792D_FAN2_MIN 42 /* RW */
+#define SENSORS_W83792D_FAN3_MIN 43 /* RW */
+#define SENSORS_W83792D_FAN4_MIN 44 /* RW */
+#define SENSORS_W83792D_FAN5_MIN 45 /* RW */
+#define SENSORS_W83792D_FAN6_MIN 46 /* RW */
+#define SENSORS_W83792D_FAN7_MIN 47 /* RW */
+
+#define SENSORS_W83792D_TEMP1 51      /* R */
+#define SENSORS_W83792D_TEMP1_HYST 52 /* RW */
+#define SENSORS_W83792D_TEMP1_OVER 53 /* RW */
+#define SENSORS_W83792D_TEMP2 54      /* R */
+#define SENSORS_W83792D_TEMP2_HYST 55 /* RW */
+#define SENSORS_W83792D_TEMP2_OVER 56 /* RW */
+#define SENSORS_W83792D_TEMP3 57      /* R */
+#define SENSORS_W83792D_TEMP3_HYST 58 /* RW */
+#define SENSORS_W83792D_TEMP3_OVER 59 /* RW */
+/*#define SENSORS_W83792D_VID 61 */   /* R */
+/*#define SENSORS_W83792D_VRM 62 */   /* RW */
+#define SENSORS_W83792D_CHASSIS 63    /* R */
+
+#define SENSORS_W83792D_FAN1_DIV 71 /* RW */
+#define SENSORS_W83792D_FAN2_DIV 72 /* RW */
+#define SENSORS_W83792D_FAN3_DIV 73 /* RW */
+#define SENSORS_W83792D_FAN4_DIV 74 /* RW */
+#define SENSORS_W83792D_FAN5_DIV 75 /* RW */
+#define SENSORS_W83792D_FAN6_DIV 76 /* RW */
+#define SENSORS_W83792D_FAN7_DIV 77 /* RW */
+
+
 
 #define SENSORS_AS99127F_PREFIX "as99127f"
 
@@ -749,6 +843,36 @@
 #define SENSORS_W83L785TS_TEMP_OVER	52 /* R for now */
 
 
+/* Winbond W83627EHF Super-I/O chips */
+
+#define SENSORS_W83627EHF_PREFIX	"w83627ehf"
+
+#define SENSORS_W83627EHF_FAN1		31 /* R  */
+#define SENSORS_W83627EHF_FAN2		32 /* R  */
+#define SENSORS_W83627EHF_FAN3		33 /* R  */
+#define SENSORS_W83627EHF_FAN4		34 /* R  */
+#define SENSORS_W83627EHF_FAN5		35 /* R  */
+#define SENSORS_W83627EHF_FAN1_MIN	41 /* RW */
+#define SENSORS_W83627EHF_FAN2_MIN	42 /* RW */
+#define SENSORS_W83627EHF_FAN3_MIN	43 /* RW */
+#define SENSORS_W83627EHF_FAN4_MIN	44 /* RW */
+#define SENSORS_W83627EHF_FAN5_MIN	45 /* RW */
+#define SENSORS_W83627EHF_FAN1_DIV	51 /* RW */
+#define SENSORS_W83627EHF_FAN2_DIV	52 /* RW */
+#define SENSORS_W83627EHF_FAN3_DIV	53 /* RW */
+#define SENSORS_W83627EHF_FAN4_DIV	54 /* RW */
+#define SENSORS_W83627EHF_FAN5_DIV	55 /* RW */
+#define SENSORS_W83627EHF_TEMP1		61 /* R  */
+#define SENSORS_W83627EHF_TEMP2		62 /* R  */
+#define SENSORS_W83627EHF_TEMP3		63 /* R  */
+#define SENSORS_W83627EHF_TEMP1_OVER	71 /* RW */
+#define SENSORS_W83627EHF_TEMP2_OVER	72 /* RW */
+#define SENSORS_W83627EHF_TEMP3_OVER	73 /* RW */
+#define SENSORS_W83627EHF_TEMP1_HYST	81 /* RW */
+#define SENSORS_W83627EHF_TEMP2_HYST	82 /* RW */
+#define SENSORS_W83627EHF_TEMP3_HYST	83 /* RW */
+
+
 /* Analog Devices ADM9240 chips */
 
 #define SENSORS_ADM9240_PREFIX "adm9240"
@@ -829,17 +953,17 @@
 #define SENSORS_SIS5595_IN1 2 /* R */
 #define SENSORS_SIS5595_IN2 3 /* R */
 #define SENSORS_SIS5595_IN3 4 /* R */
-#define SENSORS_SIS5595_IN4 4 /* R */
+#define SENSORS_SIS5595_IN4 5 /* R */
 #define SENSORS_SIS5595_IN0_MIN 11 /* RW */
 #define SENSORS_SIS5595_IN1_MIN 12 /* RW */
 #define SENSORS_SIS5595_IN2_MIN 13 /* RW */
 #define SENSORS_SIS5595_IN3_MIN 14 /* RW */
-#define SENSORS_SIS5595_IN4_MIN 14 /* RW */
+#define SENSORS_SIS5595_IN4_MIN 15 /* RW */
 #define SENSORS_SIS5595_IN0_MAX 21 /* RW */
 #define SENSORS_SIS5595_IN1_MAX 22 /* RW */
 #define SENSORS_SIS5595_IN2_MAX 23 /* RW */
 #define SENSORS_SIS5595_IN3_MAX 24 /* RW */
-#define SENSORS_SIS5595_IN4_MAX 24 /* RW */
+#define SENSORS_SIS5595_IN4_MAX 25 /* RW */
 #define SENSORS_SIS5595_FAN1 31 /* R */
 #define SENSORS_SIS5595_FAN2 32 /* R */
 #define SENSORS_SIS5595_FAN1_MIN 41 /* RW */
@@ -1986,4 +2110,16 @@
 #define SENSORS_MAX6650_FAN4_TACH  4 /* R */
 #define SENSORS_MAX6650_SPEED      5 /* RW */
 
+/* SMSC47B397-NC chip */
+#define SENSORS_SMSC47B397_PREFIX "smsc47b397"
+
+#define SENSORS_SMSC47B397_TEMP1	0x01 /* R */
+#define SENSORS_SMSC47B397_TEMP2	0x02 /* R */
+#define SENSORS_SMSC47B397_TEMP3	0x03 /* R */
+#define SENSORS_SMSC47B397_TEMP4	0x04 /* R */
+#define SENSORS_SMSC47B397_FAN1		0x11 /* R */
+#define SENSORS_SMSC47B397_FAN2		0x12 /* R */
+#define SENSORS_SMSC47B397_FAN3		0x13 /* R */
+#define SENSORS_SMSC47B397_FAN4		0x14 /* R */
+
 #endif /* def LIB_SENSORS_CHIPS_H */
