@@ -35,19 +35,13 @@
 #define VERSION LM_VERSION
 #define DEFAULT_CONFIG_FILE_NAME "sensors.conf"
 
-static char *config_file_name = NULL;
 FILE *config_file;
-static const char *config_file_path[] = 
-  { "/etc", "/usr/local/etc", "/usr/lib/sensors", "/usr/local/lib/sensors",
-    "/usr/lib", "/usr/local/lib", ".", 0 };
 extern const char *libsensors_version;
 
 extern int main(int argc, char *arv[]);
 static void print_short_help(void);
 static void print_long_help(void);
 static void print_version(void);
-static void open_config_file(void);
-static int open_this_config_file(char *filename);
 static void do_a_print(sensors_chip_name name);
 static int do_a_set(sensors_chip_name name);
 static int do_the_real_work(int *error);
@@ -68,7 +62,7 @@ void print_short_help(void)
 void print_long_help(void)
 {
   printf("Usage: %s [OPTION]... [CHIP]...\n",PROGRAM);
-  printf("  -c, --config-file     Specify a config file\n");
+  printf("  -c, --config-file     Specify a config file (default: " ETCDIR "/" DEFAULT_CONFIG_FILE_NAME ")\n");
   printf("  -h, --help            Display this help text\n");
   printf("  -s, --set             Execute `set' statements too (root only)\n");
   printf("  -f, --fahrenheit      Show temperatures in degrees fahrenheit\n");
@@ -78,7 +72,6 @@ void print_long_help(void)
   printf("  -u, --unknown         Treat chips as unknown ones (testing only)\n");
   printf("  -v, --version         Display the program version\n");
   printf("\n");
-  printf("By default, a list of directories is examined for config file `sensors.conf'.\n");
   printf("Use `-' after `-c' to read the config file from stdin.\n");
   printf("If no chips are specified, all chip info will be printed.\n");
   printf("Example chip names:\n");
@@ -98,62 +91,27 @@ void print_version(void)
 
 /* This examines global var config_file, and leaves the name there too. 
    It also opens config_file. */
-void open_config_file(void)
+void open_config_file(const char* config_file_name)
 {
-#define MAX_FILENAME_LEN 1024
-  char *filename;
-  char buffer[MAX_FILENAME_LEN];
-  int res,i;
-
-  if (config_file_name && !strcmp(config_file_name,"-")) {
+  if (!strcmp(config_file_name,"-")) {
     config_file = stdin;
     return;
-  } else if (config_file_name && index(config_file_name,'/')) {
-    if ((res = open_this_config_file(config_file_name))) {
-      fprintf(stderr,"Could not locate or open config file\n");
-      fprintf(stderr,"%s: %s\n",config_file_name,strerror(-res));
-      exit(1);
-    }
   }
-  else {
-    if (config_file_name)
-      filename = config_file_name;
-    else
-      filename = DEFAULT_CONFIG_FILE_NAME;
-    for (i = 0; config_file_path[i]; i++) {
-      if ((snprintf(buffer,MAX_FILENAME_LEN,
-                   "%s/%s",config_file_path[i],filename)) < 1) {
-        fprintf(stderr,
-                "open_config_file: ridiculous long config file name!\n");
-        exit(1);
-      }
-      if (!open_this_config_file(buffer)) {
-        free(config_file_name);
-        config_file_name = strdup(buffer);
-        return;
-      }
-    }
-    fprintf(stderr,"Could not locate or open config file!\n");
+
+  config_file = fopen(config_file_name, "r");
+  if (!config_file) {
+    fprintf(stderr, "Could not open config file\n");
+    perror(config_file_name);
     exit(1);
   }
 }
     
-int open_this_config_file(char *filename)
+void close_config_file(const char* config_file_name)
 {
-  config_file = fopen(filename,"r");
-  if (! config_file)
-    return -errno;
-  return 0;
-}
-
-void close_config_file(void)
-{
-  if (fclose(config_file)) {
+  if (fclose(config_file) == EOF) {
     fprintf(stderr,"Could not close config file\n");
-    fprintf(stderr,"%s: %s\n",config_file_name,strerror(errno));
+    perror(config_file_name);
   }
-  
-  free(config_file_name);
 }
 
 static void set_degstr(void)
@@ -184,6 +142,7 @@ static void set_degstr(void)
 int main (int argc, char *argv[])
 {
   int c,res,i,error;
+  char *config_file_name = NULL;
 
   struct option long_opts[] =  {
     { "help", no_argument, NULL, 'h' },
@@ -264,7 +223,9 @@ int main (int argc, char *argv[])
       }
 
 
-  open_config_file();
+  if (config_file_name == NULL)
+    config_file_name = strdup(ETCDIR "/" DEFAULT_CONFIG_FILE_NAME);
+  open_config_file(config_file_name);
 
   if ((res = sensors_init(config_file))) {
     fprintf(stderr,"%s\n",sensors_strerror(res));
@@ -277,7 +238,8 @@ int main (int argc, char *argv[])
     exit(1);
   }
 
-  close_config_file();
+  close_config_file(config_file_name);
+  free(config_file_name);
 
   /* build the degrees string */
   set_degstr();
@@ -328,6 +290,9 @@ int do_a_set(sensors_chip_name name)
               sensors_strerror(res));
       fprintf(stderr,"Run as root?\n");
       return 1;
+    } else if (res == -SENSORS_ERR_ACCESS_W) {
+      fprintf(stderr, "%s: At least one \"set\" statement failed\n",
+              sprintf_chip_name(name));
     } else {
       fprintf(stderr,"%s: %s\n",sprintf_chip_name(name),
               sensors_strerror(res));
@@ -382,8 +347,10 @@ struct match matches[] = {
 	{ "adm1027", print_lm85 },
 	{ "adt7463", print_lm85 },
 	{ "emc6d100", print_lm85 },
+	{ "emc6d102", print_lm85 },
 	{ "lm87", print_lm87 },
 	{ "gl518sm", print_gl518 },
+	{ "gl520sm", print_gl520 },
 	{ "adm1025", print_adm1025 },
 	{ "ne1619", print_adm1025 },
 	{ "adm1024", print_adm1024 },
@@ -394,7 +361,9 @@ struct match matches[] = {
 	{ "w83627thf", print_w83781d },
 	{ "w83637hf", print_w83781d },
 	{ "w83697hf", print_w83781d },
+	{ "w83627ehf", print_w83627ehf },
 	{ "w83791d", print_w83781d },
+        { "w83792d", print_w83792d },
 	{ "w83l785ts", print_w83l785ts },
 	{ "as99127f", print_w83781d },
 	{ "maxilife", print_maxilife },
@@ -403,6 +372,7 @@ struct match matches[] = {
 	{ "maxilife-as", print_maxilife },
 	{ "maxilife-nba", print_maxilife },
 	{ "it87", print_it87 },
+	{ "it8712", print_it87 },
 	{ "ddcmon", print_ddcmon },
 	{ "eeprom", print_eeprom },
 	{ "fscpos", print_fscpos },
@@ -424,11 +394,17 @@ struct match matches[] = {
 	{ "lm90", print_lm90 },
 	{ "adm1032", print_lm90 },
 	{ "lm99", print_lm90 },
+	{ "lm86", print_lm90 },
+	{ "max6657", print_lm90 },
+	{ "adt7461", print_lm90 },
+	{ "lm63", print_lm63 },
 	{ "xeontemp", print_xeontemp },
 	{ "max6650", print_max6650 },
 	{ "asb100", print_asb100 },
 	{ "adm1030", print_adm1031 },
 	{ "adm1031", print_adm1031 },
+	{ "lm93", print_lm93 },
+	{ "smsc47b397", print_smsc47b397 },
 	{ NULL, NULL }
 };
 
